%% STAGE 0: SETTINGS

clear
fig = 1; % 1 to re-generate figures
save('setting.mat','fig','-append');
%#ok<*ASGLU> 
%#ok<*NODEF>
%#ok<*NASGU>
%#ok<*INUSD>
%#ok<*USENS>
%#ok<*IDISVAR>
%#ok<*AGROW>

%% STAGE 1: DATA
    % LOAD THE DATA
    % MANAGE THE DATA
    % VISUALISE THE DATA

% DATA Loading
%/ annual-age data
[varvalue, varname, ~] = xlsread('../data/data.xlsx','mortality.HMD','R:U'); 
for ii = 1:length(varname) % assigning varnames to each column vector
    eval([char(varname(ii)), ' = varvalue(:,', num2str(ii), ');']);
end
%/ monthly data
[varvalue, varname, ~] = xlsread('../data/data.xlsx','monthly',''); 
for ii = 1:length(varname) % assigning varnames to each column vector
    eval([char(varname(ii)), ' = varvalue(:,', num2str(ii), ');']);
end
%/ annual data
[varvalue, varname, ~] = xlsread('../data/data.xlsx','annual',''); 
for ii = 1:length(varname) % assigning varnames to each column vector
    eval([char(varname(ii)), ' = varvalue(:,', num2str(ii), ');']);
end
%/ 5-year data
[varvalue, varname, ~] = xlsread('../data/data.xlsx','mortality.WS','A:B'); 
for ii = 1:length(varname) % assigning varnames to each column vector
    eval([char(varname(ii)), ' = varvalue(:,', num2str(ii), ');']);
end
level_WS = [0,xlsread('../data/data.xlsx','mortality.WS','E22:M22')]; % life expectancy from WS
level_WS25 = [25,xlsread('../data/data.xlsx','mortality.WS','Y22:AG22')]; % life expectancy from WS conditional on surviving 25
m_WS = [ones(5,1),xlsread('../data/data.xlsx','mortality.WS','O30:W34')]; % mortality rates from WS; add upper limit of mortality rates in column 1
lexp_WS = nan(size(lexp1));
for ii = 1:length(year_5y)
    lexp_WS(yyyy==year_5y(ii),1) = lexp2(ii); % 1541-1871
end
lexp_WS = fillmissing(lexp_WS,'previous'); % fill all missing values by previous obs
lexp_WS(find(yyyy==max(year_5y),1)+1:end) = NaN; % reset the out-of-sample values to missing
%/ decade data
[varvalue, varname, ~] = xlsread('../data/data.xlsx','decade',''); 
varvalue = varvalue(:,1:find(strcmp(varname,'XXXX'))-1); % remove all the columns beyond XXXX indicator
varname = varname(1:find(strcmp(varname,'XXXX'))-1); % remove all the columns beyond XXXX indicator
for ii = 1:length(varname) % assigning varnames to each column vector
    eval([char(varname(ii)), ' = varvalue(:,', num2str(ii), ');']);
end
%/ other data (Russell + Hatcher)
[varvalue, varname, ~] = xlsread('../data/data.xlsx','mortality.R','A1:R9'); 
for ii = 1:length(varname) % assigning varnames to each column vector
    eval([char(varname(ii)), ' = varvalue(:,', num2str(ii), ');']);
end
[varvalue, varname, ~] = xlsread('../data/data.xlsx','mortality.H','A1:C13'); 
for ii = 1:length(varname) % assigning varnames to each column vector
    eval([char(varname(ii)), ' = varvalue(:,', num2str(ii), ');']);
end

% DATA Management
YY = size(yyyy,1); % number of years in the data

% Death/Mortality Rates
%/ source 1: HMD
Nyear_AA = max(dyear)-min(dyear)+1; % total number of observed years of annual-age data
Lifespan = max(dage); % longest lifespan
dr_year = reshape(dr,Lifespan+1,Nyear_AA)/100000; % *unconditional* distribution of deaths (age-by-year), NB: 100000 is the starting population
mr_year = reshape(mr,Lifespan+1,Nyear_AA); % *conditional* distribution of deaths (age-by-year)
d0 = (sum(dr_year(1:16,:)))'; % death share of 0-15
d1 = (sum(dr_year(17:31,:)))'; % death share of 16-30
d2 = (sum(dr_year(32:46,:)))'; % death share of 31-45
d3 = (sum(dr_year(47:61,:)))'; % death share of 46-60
d4 = 1-d0-d1-d2-d3; % death share of 61+
m0_HMD = matchfill(d0,(min(dyear):max(dyear))',yyyy); % mortality rate of 0-15
m1_HMD = matchfill(d1./(1-d0),(min(dyear):max(dyear))',yyyy); % mortality rate of 16-30
m2_HMD = matchfill(d2./(1-d0-d1),(min(dyear):max(dyear))',yyyy); % mortality rate of 31-45
m3_HMD = matchfill(d3./(1-d0-d1-d2),(min(dyear):max(dyear))',yyyy); % mortality rate of 46-60
m4_HMD = matchfill((sum(dr_year(62:76,:)))'./d4,(min(dyear):max(dyear))',yyyy); % mortality rate of 61-75 (m4 = m5 & m6 = 100%)
%/ source 2: WS
lexp_index = nan(YY,length(level_WS)); % level index for the lower lexp
for ii = find(yyyy==min(year_5y),1):find(yyyy==max(year_5y),1)
    lexp_index(ii,:) = lexp_WS(ii)<level_WS;
end
lexp_index = sum(lexp_index==0,2); % find the index for the lower lexp
overlap = find(~isnan(lexp1),1,'first'):find(~isnan(lexp_WS),1,'last'); % find the overlapping periods btw lexp and lexp_WS
m0_WS = nan(YY,1);
m1_WS = nan(YY,1);
m2_WS = nan(YY,1);
m3_WS = nan(YY,1);
m4_WS = nan(YY,1);
for ii = find(yyyy==min(year_5y),1):find(yyyy==max(year_5y),1)
    if lexp_index(ii)>=2 && lexp_index(ii)<=9 % weight between the two neighbouring levels
        lexp_weight = 1-(level_WS(lexp_index(ii)+1)-lexp_WS(ii))/(level_WS(lexp_index(ii)+1)-level_WS(lexp_index(ii)));
    elseif lexp_index(ii)==10 % if at the top level
        lexp_weight = 0;
    elseif lexp_index(ii)==1 % if at the bottom level
        lexp_weight = 1;
    end
    m0_WS(ii) = (1-lexp_weight)*m_WS(1,lexp_index(ii))+lexp_weight*m_WS(1,lexp_index(ii)+1);
    m1_WS(ii) = (1-lexp_weight)*m_WS(2,lexp_index(ii))+lexp_weight*m_WS(2,lexp_index(ii)+1);
    m2_WS(ii) = (1-lexp_weight)*m_WS(3,lexp_index(ii))+lexp_weight*m_WS(3,lexp_index(ii)+1);
    m3_WS(ii) = (1-lexp_weight)*m_WS(4,lexp_index(ii))+lexp_weight*m_WS(4,lexp_index(ii)+1);
    m4_WS(ii) = (1-lexp_weight)*m_WS(5,lexp_index(ii))+lexp_weight*m_WS(5,lexp_index(ii)+1);
end
m0_WS = m0_WS/(nanmean(m0_WS(overlap)./m0_HMD(overlap))); % adjustment based on HMD
m1_WS = m1_WS/(nanmean(m1_WS(overlap)./m1_HMD(overlap)));
m2_WS = m2_WS/(nanmean(m2_WS(overlap)./m2_HMD(overlap)));
m3_WS = m3_WS/(nanmean(m3_WS(overlap)./m3_HMD(overlap)));
m4_WS = m4_WS/(nanmean(m4_WS(overlap)./m4_HMD(overlap)));
%/ source 3: Russell
SS0_0 = 1000*(1-m0_0/1000); % No of survivals per 1000 after 0
for ii = 0:4
    for jj = 1:3
        eval(['SS',num2str(ii),'_',num2str(jj),' = SS',num2str(ii),'_',num2str(jj-1),'.*((1-m',num2str(ii),'_',num2str(jj),'/1000).^5);'])
    end
    eval(['SS',num2str(ii+1),'_0 = SS',num2str(ii),'_3;'])
end
m_R = [(1000-SS0_3)/1000,(SS0_3-SS1_3)./SS0_3,(SS1_3-SS2_3)./SS1_3,(SS2_3-SS3_3)./SS2_3,(SS3_3-SS4_3)./SS3_3];
m0_R = nan(YY,1);
m1_R = nan(YY,1);
m2_R = nan(YY,1);
m3_R = nan(YY,1);
m4_R = nan(YY,1);
for ii = 1:size(t_starting,1)
    rows_R = yyyy<=t_ending(ii) & yyyy>=t_starting(ii);
    for jj = 0:4
        eval(['m',num2str(jj),'_R(rows_R) = m_R(',num2str(ii),',',num2str(jj+1),');'])
    end
end
%/ source 4: Hatcher
tT_H = (min(t_startingH):max(t_endingH))'; % sample range
lexp_H0 = nan(size(tT_H,1),size(lexp3,1));
for ii = 1:size(lexp3,1)
    lexp_H0(tT_H>=t_startingH(ii) & tT_H<=t_endingH(ii),ii) = lexp3(ii);
end
lexp_H = matchfill(nanmean(lexp_H0,2),tT_H,yyyy); % average life expectancy
lexp_index = nan(YY,length(level_WS25)); % level index for the lower lexp
for ii = find(yyyy==min(t_startingH),1):find(yyyy==max(t_endingH),1)
    lexp_index(ii,:) = lexp_H(ii)<level_WS25;
end
lexp_index = sum(lexp_index==0,2); % find the index for the lower level
overlap = find(~isnan(m_R),1,'first'):find(~isnan(lexp_H),1,'last'); % find the overlapping periods btw Russell and Hatcher
m0_H = nan(YY,1);
m1_H = nan(YY,1);
m2_H = nan(YY,1);
m3_H = nan(YY,1);
m4_H = nan(YY,1);
for ii = find(yyyy==min(t_startingH),1):find(yyyy==max(t_endingH),1)
    if lexp_index(ii)>=2 && lexp_index(ii)<=9 % weight between the two neighbouring levels
        lexp_weight = 1-(level_WS25(lexp_index(ii)+1)-lexp_H(ii))/(level_WS25(lexp_index(ii)+1)-level_WS25(lexp_index(ii)));
    elseif lexp_index(ii)==10 % if at the top level
        lexp_weight = 0;
    elseif lexp_index(ii)==1 % if at the bottom level
        lexp_weight = 1;
    end
    m0_H(ii) = (1-lexp_weight)*m_WS(1,lexp_index(ii))+lexp_weight*m_WS(1,lexp_index(ii)+1);
    m1_H(ii) = (1-lexp_weight)*m_WS(2,lexp_index(ii))+lexp_weight*m_WS(2,lexp_index(ii)+1);
    m2_H(ii) = (1-lexp_weight)*m_WS(3,lexp_index(ii))+lexp_weight*m_WS(3,lexp_index(ii)+1);
    m3_H(ii) = (1-lexp_weight)*m_WS(4,lexp_index(ii))+lexp_weight*m_WS(4,lexp_index(ii)+1);
    m4_H(ii) = (1-lexp_weight)*m_WS(5,lexp_index(ii))+lexp_weight*m_WS(5,lexp_index(ii)+1);
end
m0_H = m0_H/(nanmean(m0_H(overlap)./m0_R(overlap))); % adjustment based on Russell
m1_H = m1_H/(nanmean(m1_H(overlap)./m1_R(overlap)));
m2_H = m2_H/(nanmean(m2_H(overlap)./m2_R(overlap)));
m3_H = m3_H/(nanmean(m3_H(overlap)./m3_R(overlap)));
m4_H = m4_H/(nanmean(m4_H(overlap)./m4_R(overlap)));
%/ interpolation
for ii = 0:4
    eval(['m',num2str(ii),' = nanmean([m',num2str(ii),'_HMD,m',num2str(ii),'_WS,m',num2str(ii),'_R,m',num2str(ii),'_H],2);']);
    eval(['m',num2str(ii),' = interp1(yyyy(~isnan(m',num2str(ii),')),m',num2str(ii),'(~isnan(m',num2str(ii),')),yyyy,"linear","extrap");'])
end

% Earnings and Wages
%/ Female Nominal Wage (in new pence)
fnwage_d2a = matchfill(kron((0.7*fnwageHW15_C+0.3*fnwageHW15_A)/10,ones(10,1)),(min(year_starting):max(year_ending))',yyyy); % 1260-1850: nominal hourly wage in (new) pence assuming 10 hours a day and 30% single for A 70% married for C
fnwage_m2a = matchfill(grpstats(fnwage_level,year_monthly,{'mean'}),(min(year_monthly):max(year_monthly))',yyyy)/2.4; % 1938-1968: convert monthly data to annual data; old pence to new pence (240::100)
fnwage_spl = nanmean([fnwage1,0.4*fnwage3+0.6*interp1(yyyy(~isnan(fnwage4)),fnwage4(~isnan(fnwage4)),yyyy)],2)*fnwage_m2a(yyyy==1938)/fnwage1(yyyy==1938); % 1875-1938: convert index to pence by splicing at 1938; old pence to new pence (240::100); domestic 40% industrial 60%
fnwage = nanmean([fnwage_d2a,fnwage_m2a,fnwage_spl,fnwage2],2); % combine series
%/ Male Nominal Wage (in new pence)
mnwage_d2a = matchfill(kron(mnwageHW15_C/10,ones(10,1)),(year_starting(1):year_ending(end))',yyyy); % 1260-1850: nominal hourly wage in (new) pence assuming 10 hours a day
mnwage_m2a = matchfill(grpstats(mnwage_level,year_monthly,{'mean'}),(min(year_monthly):max(year_monthly))',yyyy)/2.4; % 1938-1968: convert monthly data to annual data; old pence to new pence (240::100)
last_obs = find(~isnan(mnwageM_B),true,'last'); % last observation of mnwageM_B
mnwageM = mnwageM_B; 
mnwageM(last_obs:end) = mnwageM_C(last_obs:end)*mnwageM_B(last_obs)/mnwageM_C(last_obs);
mnwageM = mnwageM*mnwage_d2a(yyyy==1849)/100; % 1850-1936: nominal wage in (new) pence by splicing at 1849
mnwage = nanmean([mnwage_d2a,mnwage_m2a,mnwageM,mnwage2],2); % combine series
%/ Male/Female Wage Premium (in decimal)
WP_point = log(mnwage./fnwage); % take ratio only (benchmark points)
WP = log(interp1(yyyy(~isnan(mnwage)),mnwage(~isnan(mnwage)),yyyy)./interp1(yyyy(~isnan(fnwage)),fnwage(~isnan(fnwage)),yyyy)); % interpolation first, then take ratio
WP(isnan(WP)) = WP(find(~isnan(WP),1));
%/ Real Wage & Real Wage Indices by Gender (2010 female wage = 100)
frwage = fnwage./price; % raw female real wage index
frwage_index = 100*frwage/frwage(yyyy==2010); % nomalised female real wage index
mrwage = mnwage./price; % raw male real wage index
mrwage_index = 100*mrwage/frwage(yyyy==2010); % nomalised male real wage index
%/ Female Participation Rate (in decimal)
fpart1_interp = interp1(yyyy(~isnan(fpart1)),fpart1(~isnan(fpart1)),yyyy); % fpart1 interpolated
fpart2_interp = interp1(yyyy(~isnan(fpart2)),fpart2(~isnan(fpart2)),yyyy); % fpart2 interpolated
fpart = nanmean([fpart1_interp,fpart2_interp],2); % female participation rate (in decimal)
fpart = fillmissing(fpart,'next');
%/ Real Earning Index & Real GDP per Capita Index (2010 = 100)
w = 0.75*(rearnings/(1.25/2)) + 0.25*((0.3*300+0.7*(6*17)*fpart).*frwage*10); % Clark MW male wage and HH female wage up to 1850
w(find(yyyy==1851):end) = rearnings(find(yyyy==1851):end); % restore the earnings back to Clark's after 1851
w(1:find(yyyy==1850)) = w(1:find(yyyy==1850))*rearnings(yyyy==1850)/w(yyyy==1850);
w_index = 100*w/w(yyyy==2010); % normalised real earnings index
RGDP_index = 100*RGDP/RGDP(yyyy==2010); % nomalised real GDP per capita index

% Demography
%/ Population Index (pre-1780 average = 1)
P_bar = nanmean(pop(yyyy<=1780)); % the equilibrium population under Agricultural Equilibrium
pop_index = 100*pop/pop(yyyy==2010); % nomalised population
%/ Generational Structure
P1 = P1./pop;
P2 = P2./pop;
P3 = P3./pop;
P4 = P4./pop;
%/ Birth Rate (in decimal)
birth = splice([birth1,birth2],1); % births by splicing
CBR = birth./pop; % crude birth rate (in decimal)
NPW = GFR*(45-15)/1000; % total number of births per woman 15-45
%/ Death Rate (in decimal)
overlap = logical(~isnan(death1).*~isnan(death2)); % the overlapped observations
death = [death1(1:find(overlap==1,1,'last'));death2(find(overlap==1,1,'last')+1:end)*(mean(death1(overlap)./death2(overlap)))]; % deaths by splicing
CDR = death./(pop+birth); % crude death rate including newborn (in decimal)
%/ Age of First-Time Mothers (in years)
A_marriage = nanmean([A_marriage1,A_marriage2],2);
A_first = A_marriage+mean(A_birth(find(yyyy==1938):find(yyyy==1980))-A_marriage(find(yyyy==1938):find(yyyy==1980))); % derive first-mother's age
A_first(~isnan(A_birth)) = A_birth(~isnan(A_birth)); % replace by observed first-time mother's age
A = interp1(yyyy(~isnan(A_first)),A_first(~isnan(A_first)),yyyy); % interpolated first-time mother's age
%/ Rate of Childless Women (Celibacy Rate)
miu = childless*nanmean(psingle./childless); % adjust all the childless up by single benchmarks
miu(~isnan(psingle)) = psingle(~isnan(psingle));
miu = interp1(yyyy(~isnan(miu)),miu(~isnan(miu)),yyyy);

% Technological Progress Dissemination
%/ domestic appliance technology dissemination (measured by use of washing/fridge/vaccum)
dom_wash_interp = int_ext(yyyy,dom_wash,yyyy,0,1,'usemax','line'); % washing machine
dom_frg_interp = int_ext(yyyy,dom_frg,yyyy,0,1,'uselast','line'); % fridge
dom_vac_interp = int_ext(yyyy,dom_vac,yyyy,0,1,'usemax','line'); % vacuum
dom_tech = mean([dom_wash_interp,dom_frg_interp,dom_vac_interp],2); % simple average, in decimal
X1 = int_ext(yyyy,electricity,yyyy,0,1,'usemax','line'); % electrification

%/ birth control technology dissemination (measured by illegitimate birth) 1903 = 100%
X2 = 1-nonuser; % benchmarks from Lewis-Faning (1949)
    aaa = (1-nonuser(yyyy==1910))/(nanmax(illegit(yyyy<=1910))-illegit(yyyy==1910));
    bbb = nonuser(yyyy==1910)-aaa*illegit(yyyy==1910);
X2(yyyy<1910) = 1-(aaa*illegit(yyyy<1910)+bbb); % rescale illegitimacy rate using 1910 as benchmark
X2(yyyy==1975) = 1; % set 1975 = 100%
X2 = int_ext(yyyy,X2,yyyy,0,1,'usemax','line'); % contraceptive

%/ literacy rate by gender
mlit1_interp = interp1(yyyy(~isnan(mlit1)),mlit1(~isnan(mlit1)),yyyy);
mlit2_interp = interp1(yyyy(~isnan(mlit2)),mlit2(~isnan(mlit2)),yyyy);
mlit = nanmean([mlit1_interp,mlit2_interp],2);
mlit_index = int_ext(yyyy,mlit,yyyy,0,1,'usemax','pchip'); % male literacy
flit1_interp = interp1(yyyy(~isnan(flit1)),flit1(~isnan(flit1)),yyyy);
flit2_interp = interp1(yyyy(~isnan(flit2)),flit2(~isnan(flit2)),yyyy);
flit = nanmean([flit1_interp,flit2_interp],2);
FL = int_ext(yyyy,flit,yyyy,0,1,'usemax','pchip'); % female literacy
lit_index = 0.5*(mlit_index+FL); % overall literacy index
%/ school enrollment rate
school_age_10year = sum([mpop_59,fpop_59,mpop_1014,fpop_1014],2); % decadal school age population
school_age_annual = nan(size(yyyy));
for ii = find(~isnan(school_age_10year),1,'first'):find(~isnan(school_age_10year),1,'last')
    school_age_annual(yyyy==year_starting(ii)) = school_age_10year(ii); % mid-point of the decade
end
school_age_annual(find(~isnan(school_age_annual),1,'first'):find(~isnan(school_age_annual),1,'last')) = interp1(yyyy(~isnan(school_age_annual)),school_age_annual(~isnan(school_age_annual)),yyyy(find(~isnan(school_age_annual),1,'first'):find(~isnan(school_age_annual),1,'last')));
school = pupil./school_age_10year; % school enrolment rate, in decimal (decade)
school(year_starting==1950) = 1; % school leaving age became 15 in April 1947
year_spl = year_starting(find(~isnan(school),1)); % the year used for splicing
lit_15 = lagmatrix(lit_index,-15); % literacy rate 15 years later
school_spl = lit_15(1:find(yyyy==year_spl))*(school(year_starting==year_spl)/lit_15(yyyy==year_spl)); % school enrolment rate spliced with literacy for pre-1840, in decimal
school_interp = matchfill(int_ext(year_starting(~isnan(school)),school(~isnan(school)),(year_spl:yyyy(end))',0,1,'usemax','line'),(year_starting(~isnan(school)):yyyy(end))',yyyy); % interpolated within and beyond observation periods 
S = [school_spl;school_interp(find(yyyy==year_spl)+1:end)]; % school enrolment rate
school_insp = pupil_insp./school_age_annual; % school enrolment rate, in decimal (year)
school_insp(yyyy==1950) = 1; % school leaving age became 15 in April 1947
year_spl = yyyy(find(~isnan(school_insp),1)); % the year used for splicing
school_spl = lit_15(1:find(yyyy==year_spl))*(school_insp(yyyy==year_spl)/lit_15(yyyy==year_spl)); % school enrolment rate spliced with literacy for pre-1840, in decimal
school_interp = matchfill(int_ext(yyyy(~isnan(school_insp)),school_insp(~isnan(school_insp)),(year_spl:yyyy(end))',0,1,'usemax','line'),(yyyy(~isnan(school_insp)):yyyy(end))',yyyy); % interpolated within and beyond observation periods 
S_insp = [school_spl;school_interp(find(yyyy==year_spl)+1:end)]; % inspected school enrolment rate

% / deflator
deflator = price/price(yyyy==1900); % deflator 1900==1
%/ food price
pifoodONS1 = pifoodONS/100+1; % convert from percentage to multiplier
pfoodONS_archive = pifoodONS1; % archive ONS series
pfoodONS_archive(~isnan(pifoodONS1)) = movprod(pifoodONS1(~isnan(pifoodONS1)),[YY 0],'omitnan'); % accumulative product
pfood = [pfoodONS_archive,pfoodONS];
pfood(:,3) = splice(pfood,2);
pfood(:,4) = splice([pfoodF,pfood(:,3)],2);
pfood(:,5) = splice([pfoodMR,pfood(:,4)],2);

pfood(:,6) = splice([mean([poClark,pwClark],2),pfood(:,5)],2);
price_food = pfood(:,6)/pfood(yyyy==2010,6)*100; % re-index to 2010 = 100 to be consistent with the general price index
fpratio = price_food./price; % food price over general price ratio
fpratio(isnan(fpratio)) = fpratio(find(~isnan(fpratio),1,'first')); % extrapolate the initial missings using the first observation

%/ urbanisation rate
URB = interp1(yyyy(~isnan(URB)),URB(~isnan(URB)),yyyy,'linear','extrap')/100;
%/ rent
rent_d2a = nan(size(yyyy));
for ii = find(~isnan(rent),1,'first'):find(~isnan(rent),1,'last')
    rent_d2a(yyyy==year_starting(ii)) = rent(ii); % mid-point of the decade
end
rent_d2a(find(~isnan(rent_d2a),1,'first'):find(~isnan(rent_d2a),1,'last')) = interp1(yyyy(~isnan(rent_d2a)),rent_d2a(~isnan(rent_d2a)),yyyy(find(~isnan(rent_d2a),1,'first'):find(~isnan(rent_d2a),1,'last')));
rent_H = rent1;
rent_H(~isnan(rent2)) = (rent2(~isnan(rent2))/100)*rent1(find(~isnan(rent1),1,'last')); % filling in the Holmans data
rent_H = rent_H*nanmean(rent_d2a./rent_H); % splice rent_H with rent_d2a
rent_3 = rent2(find(~isnan(rent2),1,'last'))*rent3/rent3(find(~isnan(rent3),1,'first')); % splice rent_3 with rent_2
rent_annual = nanmean([rent_H,rent_d2a,rent_3],2); % mean of all sources
rent_interp = interp1(yyyy(~isnan(rent_annual)),rent_annual(~isnan(rent_annual)),yyyy,'linear'); % linear interpolation
RRT = log(rent_interp./price); % deflated by RPI
RRT(isnan(RRT)) = RRT(find(~isnan(RRT),1,'first')); % fill previous missing values by the first available obs
%/ patent
patent(find(yyyy==1852):end) = NaN; % drop the patent values after 1852 (cost of patent changes)

% CES
g_pop = (pop-lagmatrix(pop,1))./lagmatrix(pop,1); % growth rate of population (in decimal)
mme = g_pop<0; % major mortality event defined as below the cutting point
d_tilde = zeros(size(yyyy)); % generate the excessive death rate
d_tilde(mme) = abs(g_pop(mme)); % generate the excessive death rate
%/ evolution of lowerbound s_ when mme occurs
s__ = 1; % upperbound of s
s_ = zeros(size(yyyy)); % initial lowerbound of s
s = 0.5*ones(size(yyyy)); % initial mean s
for ii = 2:length(yyyy) % evolution of s_ and s
    s_(ii) = d_tilde(ii-1)*(s__-s_(ii-1))+s_(ii-1);
    s(ii) = (s__+s_(ii))/2;
end

% Convert double to single to save space
d2s = whos;
for ii = 1:length(d2s)
      if strcmp(d2s(ii).class,'double')
          name = d2s(ii).name;
          assignin('base', name, single(evalin('base', name)));
      end
end

% Annual Data
data_annual = table(yyyy,w_index,RGDP_index,WP,fpart,fpratio,... % economic variables
                    pop,NPW,P1,P2,P3,P4,birth,death,A,miu,m0,m1,m2,m3,m4,s,... % demographic variables
                    X1,X2,S,S_insp,lit_MM,edu_MM,URB,RRT,FL); % technological variables
varnames_data = data_annual.Properties.VariableNames;
data_annual = table2timetable(data_annual,'RowTimes',datetime([yyyy,ones(length(yyyy),2)])); % convert table to timetable
varstart_index = logical(strcmp(data_annual.Properties.VariableNames,'yyyy')+strcmp(data_annual.Properties.VariableNames,'pop')); % index for variables whose starting value is to be extracted
varsum_index = logical(strcmp(data_annual.Properties.VariableNames,'birth')+strcmp(data_annual.Properties.VariableNames,'death')); % index for variables whose sum is to be extracted
varmean_index = ~(varstart_index+varsum_index); % index for variables whose mean is to be extracted
           
% 15-Year Nonstationary Data
Nperiod = floor((max(yyyy)-min(yyyy))/15)+1; % number of periods
period_index = kron((1:Nperiod)',ones(15,1)); % period index
data_15year = NaN(Nperiod,size(data_annual,2),15);
for ii = 1:15 % for each unique way of slicing
    period = period_index(ii:length(yyyy)+ii-1); % periodicity for slice ii
    data_15year(:,varstart_index,ii) = data_annual{~(period==lagmatrix(period,1)),varstart_index}; % start
    data_15year(:,varsum_index,ii) = groupsum(data_annual{:,varsum_index},period,15); % sum (adjusting for NaN and period length)
    data_15year(:,varmean_index,ii) = grpstats(data_annual{:,varmean_index},period,'mean'); % mean (adjusting for NaN)
end
data_15y = array2table(data_15year(:,:,1),'VariableNames',varnames_data); % the first slice
data_15y = table2timetable(data_15y,'RowTimes',datetime([data_15y.yyyy,ones(length(data_15y.yyyy),2)])); % convert table to timetable

% 15-Year Stationarised Data
data_s15year = data_15year; % create a vector of stationarised data
for ii = 1:size(data_s15year,3)
    data_s15year(:,strcmp(varnames_data,'w_index'),ii) = data_15year(:,strcmp(varnames_data,'w_index'),ii)./lagmatrix(data_15year(:,strcmp(varnames_data,'w_index'),ii),1); % real earnings growth rate
    data_s15year(:,strcmp(varnames_data,'RGDP_index'),ii) = data_15year(:,strcmp(varnames_data,'RGDP_index'),ii)./lagmatrix(data_15year(:,strcmp(varnames_data,'RGDP_index'),ii),1); % real GDP growth rate
    data_s15year(:,strcmp(varnames_data,'pop'),ii) = data_15year(:,strcmp(varnames_data,'pop'),ii)./lagmatrix(data_15year(:,strcmp(varnames_data,'pop'),ii),1); % population growth rate
    data_s15year(:,strcmp(varnames_data,'birth'),ii) = data_15year(:,strcmp(varnames_data,'birth'),ii)./data_15year(:,strcmp(varnames_data,'pop'),ii); % crude birth rate
    data_s15year(:,strcmp(varnames_data,'death'),ii) = data_15year(:,strcmp(varnames_data,'death'),ii)./data_15year(:,strcmp(varnames_data,'pop'),ii); % crude death rate
end
data_s15y = array2table(data_s15year(:,:,1),'VariableNames',varnames_data); % the first slice
data_s15y.Properties.VariableNames{'w_index'} = 'W'; % change variable names
data_s15y.Properties.VariableNames{'RGDP_index'} = 'Y'; % change variable names
data_s15y.Properties.VariableNames{'pop'} = 'P'; % change variable names
data_s15y.Properties.VariableNames{'birth'} = 'B'; % change variable names
data_s15y.Properties.VariableNames{'death'} = 'D'; % change variable names
data_s15y = table2timetable(data_s15y,'RowTimes',datetime([data_s15y.yyyy,ones(length(data_s15y.yyyy),2)])); % convert table to timetable

%% DATA Visualisation

% Wage/Earnings
figure;
plot(yyyy(1:find(yyyy==1851)),rearnings((1:find(yyyy==1851)))/(1.25/2))
hold on
plot(yyyy(1:find(yyyy==1851)),(0.3*300+0.7*(6*17)*fpart(1:find(yyyy==1851))).*frwage(1:find(yyyy==1851))*10)
plot(yyyy,w,'k-','LineWidth',1)
xlim([1200,2016])
legend('Clark male earning','HW female earning','Average earning')
legend('boxoff')
legend('Location','northwest')
hold off
if fig == 1
    print(gcf,'-dpng','-r800','earnings.png');
end

% Figure 0: data
figure;
yyaxis left
hold on
h1 = plot(yyyy,CBR,'-');
ylabel('Crude Birth Rate')
text(1876,0.04,'\downarrow 1876')
yyaxis right
h2 = plot(yyyy,w_index,'-');
h3 = plot(yyyy,pop_index,'k-');
text(1348,12,'\leftarrow 1348 Black Death','Rotation',90)
ylabel('Population/Earning Index')
xlim([1100,2016])
legend([h1 h2 h3],'Crude Brith Rate','Real Earnings Index','Population Index','Location','northwest')
legend('boxoff')
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig0.png');
end

% Figure 2: natural selection and evolution of s
fig2 = figure;
% Panel 1: s
axis1 = axes('Position',[0.1,0.53,0.85,0.45]);
plot(yyyy,s,'-k')
axis1.XLim = [1000,2020];
axis1.YLim = [0.4,s__];
set(axis1,'xticklabel',[])
ylabel('Mean Elasticity of Substitution')
% Panel 2: population growth
axis2 = axes('Position',[0.1,0.06,0.85,0.45]);
plot(yyyy,100*g_pop,'-k',yyyy,zeros(size(yyyy)),'--k')
text(1315,-2,'Great Famine','Rotation',90,'HorizontalAlignment','right','Color',[0.5 0.5 0.5]);
text(1350,-2,'Black Death','Rotation',90,'HorizontalAlignment','right','Color',[0.5 0.5 0.5]);
text(1400,-2,'The 2nd Pandemic','Rotation',90,'HorizontalAlignment','right','Color',[0.5 0.5 0.5]);
text(1558,-2,'Choleraic Disease','Rotation',90,'HorizontalAlignment','right','Color',[0.5 0.5 0.5]);
text(1665,-2,'Great Plague of London','Rotation',90,'HorizontalAlignment','right','Color',[0.5 0.5 0.5]);
text(1728,-2,'Great Febrile Epidemic','Rotation',90,'HorizontalAlignment','right','Color',[0.5 0.5 0.5]);
text(1918,-2,'WW1','Rotation',90,'HorizontalAlignment','right','Color',[0.5 0.5 0.5]);
text(1942,-2,'WW2','Rotation',90,'HorizontalAlignment','right','Color',[0.5 0.5 0.5]);
axis2.XLim = [1000,2020];
axis2.YLim = [-20,5];
ylabel('% Change of Population')
if fig == 1
    print(gcf,'-dpng','-r800','fig2.png');
end

% Figure 1: conditional/unconditional mortality rates
fig1 = figure;
% Mortality Rate by Age
axis1 = axes('Position',[0.1,0.56,0.39,0.42]);
for ii = 1:size(mr_year,2)
    plot(0:Lifespan,mr_year(:,ii),'LineWidth',0.1,'Color',[0 0 0]+(0.8/(Nyear_AA-1))*(ii-1));
    hold on
end
axis1.XLim = [0,Lifespan];
set(axis1,'xticklabel',[])
ylabel('Mortality rate by age')
% Death Shares by Age
axis2 = axes('Position',[0.1,0.11,0.39,0.42]);
for ii = 1:size(dr_year,2)
    plot(0:Lifespan,dr_year(:,ii),'LineWidth',0.1,'Color',[0 0 0]+(0.8/(Nyear_AA-1))*(ii-1));
    hold on
end
axis2.XLim = [0,Lifespan];
xlabel('Age')
ylabel('Death share by age')
text(20,0.04,'\downarrow world wars','Color',[0.5 0.5 0.5]);
% Mortality Rate by Generation
axis3 = axes('Position',[0.58,0.56,0.39,0.42]);
plot(yyyy,m0,'k-')
hold on
plot(yyyy,m1,'-','Color',[0.5 0.5 0.5])
plot(yyyy,m2,'k--')
plot(yyyy,m3,'--','Color',[0.5 0.5 0.5])
plot(yyyy,m4,'k:')
legend({'0~15','16~30','31~45','46~60','60+'},'Color','none','Box','off')
text(1905,0.4,'WW1','Color',[0.5 0.5 0.5]);
ylabel('Mortality rate (generation)');
axis3.XLim = [1850 2000];
axis3.YLim = [0 1];
set(axis3,'xticklabel',[]);
% Death Rate by Generation
axis4 = axes('Position',[0.58,0.11,0.39,0.42]);
fig_area = area(min(dyear):max(dyear),[d0,d1,d2,d3,d4]);
fig_area(1).FaceColor = [0 0 0];
fig_area(2).FaceColor = [0.3 0.3 0.3];
fig_area(3).FaceColor = [0.5 0.5 0.5];
fig_area(4).FaceColor = [0.8 0.8 0.8];
fig_area(5).FaceColor = [1 1 1];
xlabel('Year')
ylabel('Death share (generation)')
axis4.YLim = [0,1];
axis4.XLim = [1850 2000];
legend({'0~15','16~30','31~45','46~60','60+'},'Color','none','Box','off')
text(1905,0.75,'WW1','Color',[0.5 0.5 0.5]);
text(1946,0.3,'\leftarrow WW2','Color',[0.5 0.5 0.5]);
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig1.png');
end

fig1b = figure;
hold on
plot(data_s15y.Time,data_s15y.m0,'k-')
plot(data_s15y.Time,data_s15y.m1,'-','Color',[0.5 0.5 0.5])
plot(data_s15y.Time,data_s15y.m2,'k--')
plot(data_s15y.Time,data_s15y.m3,'--','Color',[0.5 0.5 0.5])
plot(data_s15y.Time,data_s15y.m4,'k:')
ylabel('Mortality rate by generation');
aaa = area([datetime('1341-01-01') datetime('1371-01-01')],[1 1],'FaceColor',[0.8 0.8 0.8],'LineStyle','none','HandleVisibility','off');
aaa.FaceAlpha = 0.5;
legend('m0 (0-15)','m1 (16-30)','m2 (31-45)','m3 (46-60)','m4 (61+)')
legend('boxoff')
text(datetime('1371-01-01'),0.9,'\leftarrow Black Death')
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig1b.png');
end

fig1c = figure;
plot(yyyy,m0_HMD,'-',yyyy,m0_WS,'-',yyyy,m0_R,'-',yyyy,m0_H,'-')
hold on
plot(year(data_s15y.Time),data_s15y.m0,'k-','LineWidth',1)
xlim([1200 2016])
aaa = area([year('1341-01-01') year('1371-01-01')],[0.7 0.7],'FaceColor',[0.8 0.8 0.8],'LineStyle','none');
aaa.FaceAlpha = 0.5;
text(year('1371-01-01'),0.1,'\leftarrow Black Death')
legend('HMD','WS','Russell','Hatcher','Combined','Location','northeast')
legend('boxoff')
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig1c_m0.png');
end

figWP = figure;
plot(yyyy,log([mnwage_d2a,mnwageM,mnwage_m2a,mnwage2]))
hold on
ax = gca;
ax.ColorOrderIndex = 1;
plot(yyyy,log([fnwage_d2a,fnwage_spl,fnwage_m2a,fnwage2]),'LineStyle',':')
ylabel('log(nominal wage)')
yyaxis right
ax = gca;
ax.YColor = [0 0 0];
plot(yyyy,WP,'k-','LineWidth',1)
ylabel('log(male/female)')
xlim([1200 2016])
lgd = legend({'Male: HW (2015)','Male: Mitchell (1998)','Male: BLS (1971)','Male: ONS','Female: HW (2015)','Female: Layton (1908), Crafts (1984)','Female: BLS (1971)','Female: ONS','Wage premium (right axis)'},'Location','northwest');
lgd.Box = 'off';
if fig == 1
    print(gcf,'-dpng','-r800','figWP.png');
end

figFL = figure;
plot(yyyy,[mlit1_interp,mlit2_interp])
hold on
ax = gca;
ax.ColorOrderIndex = 1;
plot(yyyy,[flit1_interp,flit2_interp],'LineStyle',':')
plot(yyyy,lit_index,'k-','LineWidth',1)
plot(yyyy,data_annual.lit_MM/100,'k--')
xlim([1200 2016])
lgd = legend('Male: Schofield (1968)','Male: De Pleijt (2016)','Female: Schofield (1968)','Female: De Pleijt (2016)','Overall Literacy Rate','Overall Literacy Rate (MM)','Location','northwest');
lgd.Box = 'off';
if fig == 1
    print(gcf,'-dpng','-r800','figFL.png');
end

figFPR = figure;
plot(1209:1914,log(pfood(find(yyyy==1209,1):find(yyyy==1914,1),6)))
hold on
plot(1800:1913,log(pfood(find(yyyy==1800,1):find(yyyy==1913,1),5)))
plot(1900:1965,log(pfood(find(yyyy==1900,1):find(yyyy==1965,1),4)))
plot(1949:2016,log(pfood(find(yyyy==1949,1):find(yyyy==2016,1),3)))
ylabel('log(food price index)')
yyaxis right
ax = gca;
ax.YColor = [0 0 0];
plot(year(data_15y.Time),data_15y.fpratio,'k-','LineWidth',1)
ylabel('Ratio between food price and general price')
xlim([1200 2016])
lgd = legend('Clark (2006)','Mitchell-Rousseaux','Feinstein (1972)','ONS','Food/PRI (right axis)','Location','northwest');
lgd.Box = 'off';
if fig == 1
    print(gcf,'-dpng','-r800','figFPR.png');
end

figURB = figure;
plot(year(data_annual.Time(185:926)), data_annual.URB(185:926),'k-',year(data_annual.Time), data_annual.URB,'k:')
xlim([1200 2016])
legend('URB: urbanization rate','Linear extrapolation','Location','northwest')
legend('boxoff')
if fig == 1
    print(gcf,'-dpng','-r800','figURB.png');
end

figS = figure;
plot(yyyy,S,'k-',yyyy,S_insp,'k:')
xlim([1200 2016])
legend('School enrolment rate','Inspected school enrolment rate','Location','northwest')
legend('boxoff')
if fig == 1
    print(gcf,'-dpng','-r800','figS.png');
end

save('data.mat','data_annual','data_15y','data_s15y','Nperiod');


%% STAGE 2: CALIBRATION
clear
load('data.mat')
% Parameter names
parnames_FIX = {'sigma_A';'sigma_miu';'sigma_Y';'sigma_Q';'sigma_pn';'sigma_pq'}; % fixed parameters (estimated otherwise)
parnames_TBE = {'a_0';'a_A';'a_b';'tau_0';'tau_miu';'tau_A';'tau_w';'alph';'bet';'theta1';'theta2';'epsilon'}; % parameters (to be estimated)
parnames_EXO = {'m0';'m1';'m2';'m3';'m4'}; % exogenous parameters
parnames_END = {'s'}; % endogenous parameters
parnames_DER = {'Phi_n';'Phi_q'}; % derived parameters (in the structural model)
parnames_AUX.Phi_n = {'WP';'S';'URB';'fpratio'}; % regressor names of equation Phi_n
parnames_AUX.Phi_q = {'FL';'S_insp';'URB'}; % regressor names of equation Phi_q
% Variable names
varnames_X = {'P';'W'}; % observable variables for SS
varnames_OBS = {'P';'W';'B';'D';'A';'miu'}; % observable variables for dynare
% Shock names
shocknames = {'eps_n';'eps_q'};
% Sample range for auxiliary regression and simulation
sample.Phi_n = 1:Nperiod; % starting from the first obs of 1200 >=9
sample.Phi_q = 1:Nperiod; % starting from 1400 >=22

% Observable steady state data
data_X = horzcat(data_s15y(:,cellstr(varnames_X))); % all trend (X) data
data_X = fillmissing(data_X,'previous');
for ii = 1:size(varnames_X,1)
    data_X{isnan(data_X{:,ii}),ii} = nanmean(data_X{:,ii}); % fill in the initial missing value by mean
end
%data_X{:,:} = hpfilter(data_X{:,:},((1/15)^2)*100); % hpfilter with smoothing paramter = 100 * (1/15)^2
data_X{:,:} = movmean(data_X{:,:},[3 3],'omitnan'); % moving average filter forward and backward 3 periods (a full life is 7 periods)
data_X{data_X.P<=1,'P'} = min(data_s15y.P(data_s15y.P>1));

% Data for LIML equation-wise estimation/calibration
d_P = data_X.P;
d_W = data_X.W;
d_A = data_s15y.A;
d_B = data_s15y.B;
d_miu = data_s15y.miu;
d_B = fillmissing(data_s15y.B,'constant',nanmean(data_s15y.B(data_s15y.Time<='1761-01-01'))); % the missing values are set as pre-modern average
    d_B(end) = NaN; % the last observation is set to the previous level
d_miu = fillmissing(data_s15y.miu,'constant',nanmean(data_s15y.miu(data_s15y.Time<='1761-01-01'))); % the missing values are set as pre-modern average
    d_miu(end) = NaN; % the last observation is set to the previous level
d_m0 = data_s15y.m0;
d_m1 = data_s15y.m1;
d_m2 = data_s15y.m2;
d_P1 = (1-d_m1).*lagmatrix((1-d_m0).*d_B./d_P,1);
d_P2 = (1-d_m2).*lagmatrix(d_P1./d_P,1);
d_b = lagmatrix((d_B./(1-d_miu))./((d_P1+d_P2)/2),-1);

% Fixed parameters
%/ A equation: lnA = a_0 + a_A.lnA(-1) + a_b.b
yyy = log(d_A);
xxx = [ones(size(yyy)),lagmatrix(yyy,1),d_b];
zzz = lagmatrix(xxx,1);
csample = (isnan(yyy)+sum(isnan(xxx),2)+sum(isnan(zzz),2))==0; % common sample
[BETA1,STE1,~,sigma_A] = tsls(yyy(csample),xxx(csample,:),zzz(csample,:));
a_0 = BETA1(1);
a_A = BETA1(2);
a_b = BETA1(3);
%/ miu equation: miu = tau_0 + tau_miu.miu(-1) + tau_A.lnA + tau_w.(W-1) + sigma_miu^2
yyy = data_s15y.miu;
xxx = [ones(size(yyy)),lagmatrix(yyy,1),log(d_A),d_W-1]; % note that W is already wage growth rate
zzz = lagmatrix(xxx,1);
csample = (isnan(yyy)+sum(isnan(xxx),2)+sum(isnan(zzz),2))==0; % common sample
[BETA2,STE2,~,sigma_miu] = tsls(yyy(csample),xxx(csample,:),zzz(csample,:));
tau_0 = BETA2(1);
tau_miu = BETA2(2);
tau_A = BETA2(3);
tau_w = BETA2(4);
%/ standard deviations
sigma_Y = nanstd(log((data_s15y.Y)./lagmatrix(data_s15y.Y,1))); % sd of RGDP
sigma_Q = sigma_Y;
sigma_pn = nanstd(log((data_s15y.W)./lagmatrix(data_s15y.W,1)));
sigma_pq = sigma_pn;

% Fixed parameters to be estimated (initial guess)
alph = 0.25;
bet = 0.25;
theta1 = 0.4;
theta2 = 0.4;
epsilon = 0.4;

% Assigning values for all fixed parameters
for ii = 1:size(parnames_FIX,1)
    par_FIX.(parnames_FIX{ii}) = eval(parnames_FIX{ii});
end
for ii = 1:size(parnames_TBE,1)
    par_TBE.(parnames_TBE{ii}) = eval(parnames_TBE{ii});
end

% Assigning values for all time-varying parameters (mortality rates and s)
par_EXO = table2struct(data_s15y(:,cellstr(parnames_EXO)),'ToScalar',true);
for ii = 1:size(parnames_EXO,1) % assigning values (m)
    assignin('caller',parnames_EXO{ii},par_EXO.(parnames_EXO{ii})(1))
end
par_END = table2struct(data_s15y(:,cellstr(parnames_END)),'ToScalar',true);
for ii = 1:size(parnames_END,1) % assigning values (s)
    assignin('caller',parnames_END{ii},par_END.(parnames_END{ii})(1))
end

% Calculate unknown steady state based on observable steady state for initial period
PX = data_X.P(1);
WX = data_X.W(1);
[~,SSX] = sss(1,par_FIX,par_TBE,par_EXO,par_END,PX,WX); % SS of initial period
SS_names = fieldnames(SSX);
for ii = 1:size(SS_names,1) % assigning values
    assignin('caller',SS_names{ii},SSX.(SS_names{ii}))
end

% Observable endogenous variables
data_OBS = data_s15y(:,cellstr(varnames_OBS)); % all observed data

% First run for period 1 (starting with steady state)
for ii = 1:size(varnames_OBS,1)
    assignin('caller',varnames_OBS{ii},data_OBS.(varnames_OBS{ii}))
end
save('data_obs.mat','P','W','B','D','A','miu');
dynare model.mod noclearall; % first run of dynare to generate m file

% Steady states of endogenous variables and derived variables
var_SS = table2timetable(array2table(nan(Nperiod,M_.orig_endo_nbr)),'RowTimes',data_X.Time); % generate a table with all the endogenous variables
var_SS.Properties.VariableNames = cellstr(M_.endo_names(1:M_.orig_endo_nbr,:));
dvar_SS = table2timetable(array2table(nan(Nperiod,2)),'RowTimes',data_X.Time);
dvar_SS.Properties.VariableNames = parnames_DER;
shock = table2timetable(array2table(nan(Nperiod,6)),'RowTimes',data_X.Time);
shock.Properties.VariableNames = cellstr(M_.exo_names);
simulation = 0; % 0 = estimation; 1 = simulation
rundynare = 0; % 0 = no need for running dynare (estimation only); 1 = run dynare code for smoothed variables and shocks

save('setting.mat','parnames_FIX','parnames_TBE','parnames_END','parnames_EXO','parnames_DER','parnames_AUX',...
     'par_FIX','par_TBE','par_END','par_EXO',...
     'varnames_X','varnames_OBS','shocknames',...
     'shock','simulation','rundynare','sample','-append');
save('data.mat','data_X','data_OBS','var_SS','dvar_SS','-append');

%% STAGE 3: ESTIMATION
%x0 = [1.98257932120283,0.420877505261430,-0.0432434856738226,-3.10134809458189,-0.901497595189316,1.00826608569526,0.405001476797159,0.303014362510995,0.226109800787918,0.354815688146630,0.537719137205502,0.206793967342846];
x0 = double([a_0,   a_A,    a_b,    tau_0,  tau_miu,tau_A,  tau_w,  alph,   bet,    theta1, theta2, epsilon]);
lb =        [0,     0,      -0.5,   -5,     -1,     0,      -5,     0,      0,      0,      0,      0];
ub =        [5,     1,      0,      5,      1,      5,      5,      1,      1,      1,      1,      1];
Aineq =     [0,     0,      0,      0,      0,      0,      0,      1,      1,      0,      0,      0;
             0,     0,      0,      0,      0,      0,      0,      0,      0,      1,      1,      0];
bineq =     [1;
             1];
optionsGA = optimoptions('ga','Display','off','InitialPopulation',x0,'UseParallel',true,'PlotFcns',{@gaplotbestf,@gaplotbestindiv});
rng('default')
tic;
[x1,fval,exitFlag,output] = ga(@objective,length(x0),Aineq,bineq,[],[],lb,ub,[],optionsGA);
fprintf('Estimation takes %f hours',toc/3600);
x0 = x0';
x1 = x1';

summary_TBE = table(parnames_TBE,x0,x1); % parameters and sparameters estimated
summary_mortality = timetable2table(data_annual({'01-Jan-1851','01-Jan-1914'},{'m0','m1','m2','m3','m4'})); % mortality drops between 1851 and 1914
save('results.mat','summary_*','-append');

%% STAGE 4: POST-ESTIMATION
clear
load('setting.mat');
load('results.mat');
load('data.mat');

% Re-run the estimated model
simulation = 0;
rundynare = 0; % obtain transition
save('setting.mat','simulation','rundynare','-append');
[gap,SS0] = objective(summary_TBE.x1);

% Derived coefficients
for ii = 1:size(parnames_DER,1)
    par_DER.(parnames_DER{ii}) = SS0.(parnames_DER{ii}); % obtain the derived time-varying parameters
end
for ii = 1:size(parnames_DER,1)
	par_AUX.xxx.(parnames_DER{ii}) = [ones(length(sample.(parnames_DER{ii})),1),data_s15y{sample.(parnames_DER{ii}),parnames_AUX.(parnames_DER{ii})}]; % regressors
    par_AUX.coeff.(parnames_DER{ii}) = regress(log(par_DER.(parnames_DER{ii})(sample.(parnames_DER{ii}))),par_AUX.xxx.(parnames_DER{ii})); % regression coeff
    par_AUX.resid.(parnames_DER{ii}) = log(par_DER.(parnames_DER{ii})(sample.(parnames_DER{ii})))-(par_AUX.xxx.(parnames_DER{ii}))*(par_AUX.coeff.(parnames_DER{ii})); % regression residual
end
save('setting.mat','SS0','par_DER','par_AUX','-append');

% Validate the model by comparing the model-SS and the data
fig4 = figure;
% Population
axis1 = axes('Position',[0.1,0.7,0.39,0.28]);
plot(data_s15y.Time,data_s15y.P,'r-',data_s15y.Time,SS0.P,'k-')
hold on
plot(data_s15y.Time,ones(size(data_s15y.Time)),'Color',[0.5 0.5 0.5])
line([datetime('1700-01-01') datetime('1700-01-01')],[0.5 1.5],'Color',[0.8 0.8 0.8])
text(datetime('1700-01-01'),1.3,'1700 \rightarrow','HorizontalAlignment','right')
aaa = area([datetime('1341-01-01') datetime('1371-01-01')],[1.5 1.5],'FaceColor',[0.8 0.8 0.8],'LineStyle','none');
aaa.FaceAlpha = 0.5;
set(axis1,'xticklabel',[])
ylim([0.5 1.5])
ylabel('Population growth')
% CBR
axis2 = axes('Position',[0.1,0.4,0.39,0.28]);
plot(data_s15y.Time,data_s15y.B,'r-',data_s15y.Time,SS0.B,'k-')
hold on
line([datetime('1870-01-01') datetime('1870-01-01')],[0 1],'Color',[0.8 0.8 0.8])
text(datetime('1870-01-01'),0.8,'1870 \rightarrow','HorizontalAlignment','right')
aaa = area([datetime('1341-01-01') datetime('1371-01-01')],[1 1],'FaceColor',[0.8 0.8 0.8],'LineStyle','none');
aaa.FaceAlpha = 0.5;
set(axis2,'xticklabel',[])
ylim([0 1])
ylabel('CBR')
% CDR
axis3 = axes('Position',[0.1,0.1,0.39,0.28]);
plot(data_s15y.Time,data_s15y.D,'r-',data_s15y.Time,SS0.D,'k-')
hold on
aaa = area([datetime('1341-01-01') datetime('1371-01-01')],[1 1],'FaceColor',[0.8 0.8 0.8],'LineStyle','none');
aaa.FaceAlpha = 0.5;
text(datetime('1371-01-01'),0.2,'\leftarrow Black Death')
ylim([0 1])
ylabel('CDR')
% Wage
axis4 = axes('Position',[0.58,0.7,0.39,0.28]);
plot(data_s15y.Time,data_s15y.W,'r-',data_s15y.Time,SS0.W,'k-')
hold on
plot(data_s15y.Time,ones(size(data_s15y.Time)),'Color',[0.5 0.5 0.5])
line([datetime('1800-01-01') datetime('1800-01-01')],[0.5 1.5],'Color',[0.8 0.8 0.8])
text(datetime('1800-01-01'),1.3,'1800 \rightarrow','HorizontalAlignment','right')
set(axis4,'xticklabel',[])
ylabel('Real wage growth')
% A
axis5 = axes('Position',[0.58,0.4,0.39,0.28]);
SS0.A(isnan(data_s15y.A)) = 21;
plot(data_s15y.Time,data_s15y.A,'r-',data_s15y.Time,SS0.A,'k-')
set(axis5,'xticklabel',[])
ylabel('Age at first marriage')
% miu
axis6 = axes('Position',[0.58,0.1,0.39,0.28]);
plot(data_s15y.Time,data_s15y.miu,'r-',data_s15y.Time,SS0.miu,'k-')
ylabel('Celibacy rate')
legend('data','model','Location','northwest')
legend('boxoff')
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig4.png');
end

%% STAGE 5: SIMULATION
clear
load('data.mat');
load('setting.mat');
load('results.mat');
TT = 52; % the first period when simulation begins
TBF = {{'m0';'m1';'m2';'m3';'m4'}    % simulation = 1 fixed mortality
       {'S','S_insp'}                % simulation = 2 fixed schooling
       {'WP'}                        % simulation = 3 fixed wage premium
       {'FL'}                        % simulation = 4 fixed female literacy
       {'fpratio'}                   % simulation = 5 fixed food price ratio
       {'s'}                         % simulation = 6 fixed CES = 0.5
       {'s'}                         % simulation = 7 fixed CES = 1
       {'m0';'m1';'m2';'m3';'m4';'S';'S_insp';'WP';'FL';'RRT';'URB'} % simulation = 8 fixed all
       {'S';'S_insp';'WP';'FL';'RRT';'URB'} % simulation = 9 % fixed all except for mortalities
       {'Phi_n'} % simulation = 10 fixed Phi_n
       {'Phi_q'} % simulation = 11 fixed Phi_q
       {'Phi_n';'Phi_q'}}; % simulation = 12 fixed Phi_n and Phi_q

% data for auxiliary regressions
data_AUX = array2table([log(par_DER.Phi_n),log(par_DER.Phi_q),data_s15y{:,{'X1','X2','WP','FL','S','S_insp','fpratio','RRT','URB'}},log(SS0.A)]);
data_AUX.Properties.VariableNames = {'lphi_n','lphi_q','X1','X2','WP','FL','S','S_insp','fpratio','RRT','URB','lnA'};
writetable(data_AUX,'data_AUX.xlsx','Sheet',1);

SS1 = nan(size(SS0,1),size(SS0,2),size(TBF,1)); % create empty matrix
NNS1 = nan(Nperiod,size(TBF,1)); % create empty matrix
for ii = 1:size(TBF,1) % simulation = ii
    simulation = ii;
    load('data.mat');
    load('setting.mat','par_AUX');
    if sum(ismember(parnames_DER,TBF{ii}))>0 % if we simulate the auxiliary regressors
        data_s15y.Phi_n = SS0.Phi_n;
        data_s15y.Phi_q = SS0.Phi_q;
    end
    %/ fix the values of par_EXO or par_AUX
    for jj = 1:size(TBF{ii},1) % fix the values at TT
        data_s15y.(TBF{ii}{jj})(TT:end) = data_s15y.(TBF{ii}{jj})(TT); % temporary variable
        if ii<5 || ii>7
            par_AUX.resid.Phi_n(find(sample.Phi_n==TT):end) = par_AUX.resid.Phi_n(sample.Phi_n==TT); % fix the shocks
            par_AUX.resid.Phi_q(find(sample.Phi_q==TT):end) = par_AUX.resid.Phi_q(sample.Phi_q==TT);
        end
    end
    %/ collect the 'new' par_EXO
    for jj = 1:size(parnames_EXO,1) % mortalities
        par_SIM.par_EXO.(parnames_EXO{jj}) = data_s15y{:,parnames_EXO{jj}}; % assign the parnames_TV
    end
    %/ derive the new Phi_n and Phi_q
    for jj = 1:size(parnames_DER,1)
        par_SIM.par_DER.(parnames_DER{jj}) = SS0.(parnames_DER{jj}); % assign original Phi_n and Phi_q
        xxx = [ones(length(sample.(parnames_DER{jj})),1),data_s15y{sample.(parnames_DER{jj}),parnames_AUX.(parnames_DER{jj})}]; % regressors
        if ii==1 % no change in Phi_n and Phi_q
            par_SIM.par_DER.(parnames_DER{jj}) = SS0.(parnames_DER{jj});
        elseif ii>1 && sum(ismember(parnames_DER,TBF{ii}))==0 % fixing regressors only
            par_SIM.par_DER.(parnames_DER{jj})(sample.(parnames_DER{jj})) = exp(xxx*(par_AUX.coeff.(parnames_DER{jj}))+par_AUX.resid.(parnames_DER{jj}));
        elseif ii>1 && sum(ismember(parnames_DER,TBF{ii}))>0 % fixing everything
            par_SIM.par_DER.(parnames_DER{jj})(sample.(parnames_DER{jj})) = data_s15y.(parnames_DER{jj})(sample.(parnames_DER{jj})); 
        end
    end    
    save('setting.mat','simulation','TT','par_SIM','-append');
    [~,model_SS,~,~,NNS1(:,simulation)] = objective(summary_TBE.x1);
    SS1(:,:,simulation) = table2array(model_SS);
end

PPP = squeeze(SS1(:,strcmp(SS0.Properties.VariableNames,'P'),:)); % collect P
WWW = squeeze(SS1(:,strcmp(SS0.Properties.VariableNames,'W'),:)); % collect W
BBB = squeeze(SS1(:,strcmp(SS0.Properties.VariableNames,'B'),:)); % collect B
nnn = squeeze(SS1(:,strcmp(SS0.Properties.VariableNames,'n'),:)); % collect n
RRR = squeeze(SS1(:,strcmp(SS0.Properties.VariableNames,'ADR'),:)); % collect ADR
mmm = squeeze(SS1(:,strcmp(SS0.Properties.VariableNames,'miu'),:)); % collect mu
save('results.mat','SS1','-append');

fig3 = figure;
hold on
D1 = log((1-SS0.miu)/(1-SS0.miu(TT))); % change in 1-mu
D2 = log((SS0.P1+SS0.P2)/(SS0.P1(TT)+SS0.P2(TT))); % change in P1+P2
D3 = log((1./((1-data_s15y.m0).*(1-data_s15y.m1)))/(1/((1-data_s15y.m0(TT))*(1-data_s15y.m1(TT))))); % change in (1-m0)(1-m1)
D4 = log(SS0.n/SS0.n(TT)); % change in TT
DB = log(SS0.B/SS0.B(TT)); % change in CBR
D_pos = [D1 D2 D3 D4];
D_pos(D_pos<=0) = 0;
D_neg = [D1 D2 D3 D4];
D_neg(D_neg>0) = 0;
plot(SS0.Time,DB,'k-','LineWidth',2)
bp = bar(SS0.Time,D_pos,'stacked','EdgeColor','none','FaceColor','flat');
bp(1).CData = 1;
bp(2).CData = 2;
bp(3).CData = 3;
bp(4).CData = 4;
alpha(0.5)
bn = bar(SS0.Time,D_neg,'stacked','EdgeColor','none','FaceColor','flat');
bn(1).CData = 1;
bn(2).CData = 2;
bn(3).CData = 3;
bn(4).CData = 4;
alpha(0.5)
ylabel('$log(X/X_{1850})$','Interpreter','latex')
lg = legend('$log(CBR/CBR_{1850})$','change due to $1-\mu$','change due to fertile woman ratio','change due to $m0$ and $m1$','change due to $n$','Location','southwest');
set(lg,'Interpreter','latex')
legend('boxoff')
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig3.png');
end

fig5 = figure;
hold on
plot(SS0.Time(TT-20:end),SS0.B(TT-20:end),'k-','LineWidth',2)
%plot(SS0.Time(TT:end),BBB(TT:end,1),'r-')
plot(SS0.Time(TT:end),BBB(TT:end,2),'k--')
plot(SS0.Time(TT:end),BBB(TT:end,3),'k-+')
plot(SS0.Time(TT:end),BBB(TT:end,5),'k-*')
plot(SS0.Time(TT:end),BBB(TT:end,9),'k-o')
plot(SS0.Time(TT:end),BBB(TT:end,8),'k-s')
ylabel('Crude Birth Rate')
lg = legend('model predictions','fixed schooling','fixed gender pay gap','fixed food price ratio','fixed all auxiliary processes','fixed all auxiliary processes and mortality','Location','southwest');
set(lg,'Interpreter','latex')
legend('boxoff')
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig5.png');
end

fig6 = figure;
plot(SS0.Time(TT-20:end),2*(SS0.n(TT-20:end)),'k-','LineWidth',2)
hold on
plot(SS0.Time(TT:end),2*nnn(TT:end,2),'k--')
plot(SS0.Time(TT:end),2*nnn(TT:end,3),'k-+')
plot(SS0.Time(TT:end),2*nnn(TT:end,5),'k-*')
plot(SS0.Time(TT:end),2*nnn(TT:end,9),'k-o','LineWidth',2)
plot(SS0.Time(TT:end),2*nnn(TT:end,8),'k-s','LineWidth',2)
lg = legend('model predictions','fixed schooling','fixed gender pay gap','fixed food price ratio','fixed all auxiliary processes','fixed all auxiliary processes and mortality','Location','northwest');
set(lg,'Interpreter','latex')
legend('boxoff')
ylabel('Target No. of Children per Mother ($2n$)','Interpreter','latex');
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig6.png');
end

fig5b = figure;
hold on
plot(SS0.Time(TT-20:end),SS0.B(TT-20:end),'k-','LineWidth',2)
plot(SS0.Time(TT:end),BBB(TT:end,10),'k.-')
plot(SS0.Time(TT:end),BBB(TT:end,11),'k--')
plot(SS0.Time(TT:end),BBB(TT:end,12),'k:')
lg = legend('model predictions','fixed $\Phi_n$','fixed $\Phi_q$','fixed $\Phi_n$ and $\Phi_q$','Location','southwest');
set(lg,'Interpreter','latex')
legend('boxoff')
ylabel('Crude Birth Rate')
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig5b.png');
end

fig6b = figure;
plot(SS0.Time(TT-20:end),2*(SS0.n(TT-20:end)),'k-','LineWidth',2)
hold on
plot(SS0.Time(TT:end),2*nnn(TT:end,10),'k.-')
plot(SS0.Time(TT:end),2*nnn(TT:end,11),'k--')
plot(SS0.Time(TT:end),2*nnn(TT:end,12),'k:')
lg = legend('model predictions','fixed $\Phi_n$','fixed $\Phi_q$','fixed $\Phi_n$ and $\Phi_q$','Location','northwest');
set(lg,'Interpreter','latex')
legend('boxoff')
ylabel('Target No. of Children per Mother ($2n$)','Interpreter','latex');
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig6b.png');
end

fig7 = figure;
hold on
plot(SS0.Time,SS0.B,'k-')
plot(SS0.Time,BBB(:,6),'k--',SS0.Time,BBB(:,7),'k:')
ylabel('Crude Birth Rate')
aaa = area([datetime('1341-01-01') datetime('1371-01-01')],[1.2 1.2],'FaceColor',[0.8 0.8 0.8],'LineStyle','none');
aaa.FaceAlpha = 0.5;
text(datetime('1371-01-01'),max(SS0.B),'\leftarrow Black Death')
line([datetime('1800-01-01') datetime('1800-01-01')],[0 1.2],'Color',[0.8 0.8 0.8])
text(datetime('1800-01-01'),max(SS0.B),'\leftarrow 1800')
line([datetime('1870-01-01') datetime('1870-01-01')],[0 1.2],'Color',[0.8 0.8 0.8])
text(datetime('1870-01-01'),1,'\leftarrow 1870')
lg = legend('model predictions','$s = 0.5$','$s = 1$','Location','southwest');
set(lg,'Interpreter','latex')
legend('boxoff')
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig7.png');
end

fig7b = figure; % simulated wage when fixing s
rwage0 = movprod(SS0.W,[height(SS0) 0]);
rwage0 = rwage0/rwage0(9); % re-indexation 1206 = 1
rwageL = movprod(WWW(:,6),[height(SS0),0]); % s = 0.5
rwageL = rwageL/rwageL(9); % re-indexation 1206 = 1
rwageU = movprod(WWW(:,7),[height(SS0),0]); % s = 1
rwageU = rwageU/rwageU(9); % re-indexation 1206 = 1
sample = 9:height(SS0); % starting from 1206
hold on
plot(SS0.Time(sample),log(rwage0(sample)),'k-')
plot(SS0.Time(sample),log(rwageL(sample)),'k--',SS0.Time(sample),log(rwageU(sample)),'k:')
ylabel('log Real Earnings Index')
aaa = area([datetime('1341-01-01') datetime('1371-01-01')],[7 7],'FaceColor',[0.8 0.8 0.8],'LineStyle','none');
aaa.FaceAlpha = 0.5;
text(datetime('1371-01-01'),4,'\leftarrow Black Death')
line([datetime('1800-01-01') datetime('1800-01-01')],[0 7],'Color',[0.8 0.8 0.8])
text(datetime('1800-01-01'),6,'\leftarrow 1800')
lg = legend('model predictions','fix $s$ at 0.5','fix $s$ at 1','Location','north');
set(lg,'Interpreter','latex')
legend('boxoff')
ylim([0 7])
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig7b.png');
end


fig8 = figure;
H_level = movprod(SS0.H,[Nperiod 0])/SS0.H(1); % human capital index (period 1 = 1)
plot(SS0.Time,log(H_level),'r-')
hold on
plot(data_annual.Time,log(data_annual.w_index),'k-')
plot(data_annual.Time,data_annual.edu_MM,'k--')
lg = legend('log level of human capital index','log level of real earnings index','years of schooling (right axis)','Location','northwest');
set(lg,'Interpreter','latex')
legend('boxoff')
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig8.png');
end

fig9 = figure;
left_color = [0 0 0];
right_color = [1 0 0];
set(fig,'defaultAxesColorOrder',[left_color; right_color]);
hold on
% yyaxis left
plot(SS0.Time,SS0.pn,'k-')
plot(SS0.Time,SS0.pq,'k--')
% yyaxis right
% plot(SS0.Time,(SS0.pn)./(SS0.pq),'r-')
lg = legend('$\pi_n$ generalized price of a surviving child','$\pi_q$ generalized price of child quality relative to parents','Location','northwest');
set(lg,'Interpreter','latex')
legend('boxoff')
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig9.png');
end

%% MONTE CARLO SIMULATION
clear
load('data.mat');
load('setting.mat');
load('results.mat');
TT = 1; % the first period when simulation begins
sim_TT = height(data_15y)-TT+1; % periods to be simulated
sim_N = 1000;
rng('default')
sim_index = randi(sim_TT,sim_TT,2,sim_N);
sim_shock = zeros(sim_TT,2,sim_N);
sim_price = zeros(sim_TT,2,sim_N);
MC = zeros(sim_TT,size(SS0,2),sim_N);
simulation = 99;
tic
for ss = 1:sim_N
    load('data.mat');
    load('setting.mat','par_AUX');
    par_SIM.par_DER.Phi_n = exp((par_AUX.xxx.Phi_n)*(par_AUX.coeff.Phi_n)+par_AUX.resid.Phi_n(sim_index(:,1,ss)));
    par_SIM.par_DER.Phi_q = exp((par_AUX.xxx.Phi_q)*(par_AUX.coeff.Phi_q)+par_AUX.resid.Phi_q(sim_index(:,2,ss)));
    save('setting.mat','simulation','TT','par_SIM','-append');
    [~,model_SS,~,~,~] = objective(summary_TBE.x1);
    MC(:,:,ss) = table2array(model_SS);
end
toc

save('results.mat','MC','-append');

PPP = squeeze(SS1(:,strcmp(SS0.Properties.VariableNames,'P'),:)); % collect P
WWW = squeeze(SS1(:,strcmp(SS0.Properties.VariableNames,'W'),:)); % collect W
BBB = squeeze(SS1(:,strcmp(SS0.Properties.VariableNames,'B'),:)); % collect B
nnn = squeeze(SS1(:,strcmp(SS0.Properties.VariableNames,'n'),:)); % collect n
RRR = squeeze(SS1(:,strcmp(SS0.Properties.VariableNames,'ADR'),:)); % collect ADR
mmm = squeeze(SS1(:,strcmp(SS0.Properties.VariableNames,'miu'),:)); % collect mu

PPP_MC = squeeze(MC(:,strcmp(SS0.Properties.VariableNames,'P'),:)); % collect P
WWW_MC = squeeze(MC(:,strcmp(SS0.Properties.VariableNames,'W'),:)); % collect W
BBB_MC = squeeze(MC(:,strcmp(SS0.Properties.VariableNames,'B'),:)); % collect B
DDD_MC = squeeze(MC(:,strcmp(SS0.Properties.VariableNames,'D'),:)); % collect D
AAA_MC = squeeze(MC(:,strcmp(SS0.Properties.VariableNames,'A'),:)); % collect A
mmm_MC = squeeze(MC(:,strcmp(SS0.Properties.VariableNames,'miu'),:)); % collect miu
nnn_MC = squeeze(MC(:,strcmp(SS0.Properties.VariableNames,'n'),:)); % collect n


% Validate the model by comparing the model-SS and the data
fig4 = figure;
% Population
axis1 = axes('Position',[0.1,0.7,0.39,0.28]);
plot(data_s15y.Time,data_s15y.P,'r-',data_s15y.Time,SS0.P,'k-')
hold on
plot(data_s15y.Time,ones(size(data_s15y.Time)),'Color',[0.5 0.5 0.5])
line([datetime('1700-01-01') datetime('1700-01-01')],[0.5 1.5],'Color',[0.4 0.4 0.4])
text(datetime('1700-01-01'),1.3,'1700 \rightarrow','HorizontalAlignment','right')
aaa = area([datetime('1341-01-01') datetime('1371-01-01')],[1.5 1.5],'FaceColor',[0.4 0.4 0.4],'LineStyle','none');
aaa.FaceAlpha = 0.5;
H = fill([SS0.Time',fliplr(SS0.Time')],[prctile(PPP_MC,5,2)',fliplr(prctile(PPP_MC,95,2)')],[0.8 0.8 0.8]);
H.EdgeAlpha = 0;
H.FaceAlpha = 0.5;
set(axis1,'xticklabel',[])
ylim([0.5 1.5])
ylabel('Population growth')
% CBR
axis2 = axes('Position',[0.1,0.4,0.39,0.28]);
plot(data_s15y.Time,data_s15y.B,'r-',data_s15y.Time,SS0.B,'k-')
hold on
line([datetime('1870-01-01') datetime('1870-01-01')],[0 1],'Color',[0.4 0.4 0.4])
text(datetime('1870-01-01'),0.8,'1870 \rightarrow','HorizontalAlignment','right')
aaa = area([datetime('1341-01-01') datetime('1371-01-01')],[1 1],'FaceColor',[0.4 0.4 0.4],'LineStyle','none');
aaa.FaceAlpha = 0.5;
H = fill([SS0.Time',fliplr(SS0.Time')],[prctile(BBB_MC,5,2)',fliplr(prctile(BBB_MC,95,2)')],[0.8 0.8 0.8]);
H.EdgeAlpha = 0;
H.FaceAlpha = 0.5;
set(axis2,'xticklabel',[])
ylim([0 1])
ylabel('CBR')
% CDR
axis3 = axes('Position',[0.1,0.1,0.39,0.28]);
plot(data_s15y.Time,data_s15y.D,'r-',data_s15y.Time,SS0.D,'k-')
hold on
aaa = area([datetime('1341-01-01') datetime('1371-01-01')],[1 1],'FaceColor',[0.4 0.4 0.4],'LineStyle','none');
aaa.FaceAlpha = 0.5;
text(datetime('1371-01-01'),0.2,'\leftarrow Black Death')
H = fill([SS0.Time',fliplr(SS0.Time')],[prctile(DDD_MC,5,2)',fliplr(prctile(DDD_MC,95,2)')],[0.8 0.8 0.8]);
H.EdgeAlpha = 0;
H.FaceAlpha = 0.5;
ylim([0 1])
ylabel('CDR')
% Wage
axis4 = axes('Position',[0.58,0.7,0.39,0.28]);
plot(data_s15y.Time,data_s15y.W,'r-',data_s15y.Time,SS0.W,'k-')
hold on
plot(data_s15y.Time,ones(size(data_s15y.Time)),'Color',[0.5 0.5 0.5])
line([datetime('1800-01-01') datetime('1800-01-01')],[0.5 1.5],'Color',[0.4 0.4 0.4])
text(datetime('1800-01-01'),1.3,'1800 \rightarrow','HorizontalAlignment','right')
H = fill([SS0.Time',fliplr(SS0.Time')],[prctile(WWW_MC,5,2)',fliplr(prctile(WWW_MC,95,2)')],[0.8 0.8 0.8]);
H.EdgeAlpha = 0;
H.FaceAlpha = 0.5;
set(axis4,'xticklabel',[])
ylabel('Real wage growth')
% A
axis5 = axes('Position',[0.58,0.4,0.39,0.28]);
SS0.A(isnan(data_s15y.A)) = 21;
plot(data_s15y.Time,data_s15y.A,'r-',data_s15y.Time,SS0.A,'k-')
hold on
H = fill([SS0.Time(~isnan(data_s15y.A))',fliplr(SS0.Time(~isnan(data_s15y.A))')],[prctile(AAA_MC(~isnan(data_s15y.A),:),5,2)',fliplr(prctile(AAA_MC(~isnan(data_s15y.A),:),95,2)')],[0.8 0.8 0.8]);
H.EdgeAlpha = 0;
H.FaceAlpha = 0.5;
set(axis5,'xticklabel',[])
ylabel('Age at first marriage')
% miu
axis6 = axes('Position',[0.58,0.1,0.39,0.28]);
plot(data_s15y.Time,data_s15y.miu,'r-',data_s15y.Time,SS0.miu,'k-')
hold on
ylabel('Celibacy rate')
H = fill([SS0.Time',fliplr(SS0.Time')],[prctile(mmm_MC,5,2)',fliplr(prctile(mmm_MC,95,2)')],[0.8 0.8 0.8]);
H.EdgeAlpha = 0;
H.FaceAlpha = 0.5;
legend('data','model','Location','northwest')
legend('boxoff')
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig4.png');
end

TT = 52;
fig5b = figure;
hold on
plot(SS0.Time(TT-20:end),SS0.B(TT-20:end),'k-','LineWidth',2)
plot(SS0.Time(TT:end),BBB(TT:end,10),'k.-')
plot(SS0.Time(TT:end),BBB(TT:end,11),'k--')
plot(SS0.Time(TT:end),BBB(TT:end,12),'k:')
H = fill([SS0.Time(TT-20:end)',fliplr(SS0.Time(TT-20:end)')],[prctile(BBB_MC(TT-20:end,:),5,2)',fliplr(prctile(BBB_MC(TT-20:end,:),95,2)')],[0.8 0.8 0.8]);
H.EdgeAlpha = 0;
H.FaceAlpha = 0.5;
lg = legend('model predictions','fixed $\Phi_n$','fixed $\Phi_q$','fixed $\Phi_n$ and $\Phi_q$','Location','southwest');
set(lg,'Interpreter','latex')
legend('boxoff')
ylabel('Crude Birth Rate')
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig5b.png');
end

fig6b = figure;
plot(SS0.Time(TT-20:end),2*(SS0.n(TT-20:end)),'k-','LineWidth',2)
hold on
plot(SS0.Time(TT:end),2*nnn(TT:end,10),'k.-')
plot(SS0.Time(TT:end),2*nnn(TT:end,11),'k--')
plot(SS0.Time(TT:end),2*nnn(TT:end,12),'k:')
H = fill([SS0.Time(TT-20:end)',fliplr(SS0.Time(TT-20:end)')],[2*prctile(nnn_MC(TT-20:end,:),5,2)',fliplr(2*prctile(nnn_MC(TT-20:end,:),95,2)')],[0.8 0.8 0.8]);
H.EdgeAlpha = 0;
H.FaceAlpha = 0.5;
lg = legend('model predictions','fixed $\Phi_n$','fixed $\Phi_q$','fixed $\Phi_n$ and $\Phi_q$','Location','northwest');
set(lg,'Interpreter','latex')
legend('boxoff')
ylabel('Target No. of Children per Mother ($2n$)','Interpreter','latex');
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig6b.png');
end

fig5 = figure;
hold on
plot(SS0.Time(TT-20:end),SS0.B(TT-20:end),'k-','LineWidth',2)
plot(SS0.Time(TT:end),BBB(TT:end,2),'k--')
plot(SS0.Time(TT:end),BBB(TT:end,3),'k-.')
plot(SS0.Time(TT:end),BBB(TT:end,4),'k-+')
plot(SS0.Time(TT:end),BBB(TT:end,5),'k-*')
plot(SS0.Time(TT:end),BBB(TT:end,9),'k-o')
plot(SS0.Time(TT:end),BBB(TT:end,8),'k-s')
H = fill([SS0.Time(TT-20:end)',fliplr(SS0.Time(TT-20:end)')],[prctile(BBB_MC(TT-20:end,:),5,2)',fliplr(prctile(BBB_MC(TT-20:end,:),95,2)')],[0.8 0.8 0.8]);
H.EdgeAlpha = 0;
H.FaceAlpha = 0.5;
ylabel('Crude Birth Rate')
lg = legend('model predictions','fixed schooling','fixed gender pay gap','fixed female literacy','fixed food price ratio','fixed all auxiliary processes','fixed all auxiliary processes and mortality','Location','southwest');
set(lg,'Interpreter','latex')
legend('boxoff')
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig5.png');
end

fig6 = figure;
plot(SS0.Time(TT-20:end),2*(SS0.n(TT-20:end)),'k-','LineWidth',2)
hold on
plot(SS0.Time(TT:end),2*nnn(TT:end,2),'k--')
plot(SS0.Time(TT:end),2*nnn(TT:end,3),'k-.')
plot(SS0.Time(TT:end),2*nnn(TT:end,4),'k-+')
plot(SS0.Time(TT:end),2*nnn(TT:end,5),'k-*')
plot(SS0.Time(TT:end),2*nnn(TT:end,9),'k-o')
plot(SS0.Time(TT:end),2*nnn(TT:end,8),'k-s')
H = fill([SS0.Time(TT-20:end)',fliplr(SS0.Time(TT-20:end)')],[2*prctile(nnn_MC(TT-20:end,:),5,2)',fliplr(2*prctile(nnn_MC(TT-20:end,:),95,2)')],[0.8 0.8 0.8]);
H.EdgeAlpha = 0;
H.FaceAlpha = 0.5;
lg = legend('model predictions','fixed schooling','fixed gender pay gap','fixed female literacy','fixed food price ratio','fixed all auxiliary processes','fixed all auxiliary processes and mortality','Location','northwest');
set(lg,'Interpreter','latex')
legend('boxoff')
ylabel('Target No. of Children per Mother ($2n$)','Interpreter','latex');
hold off
if fig == 1
    print(gcf,'-dpng','-r800','fig6.png');
end




%% LOCAL FUNCTIONS

function [y,model_SS,model_SM,model_SK,NNS,NSR] = objective(x) 
% Objective function to estimate the parameters

% Loading data/setting
load('data.mat','*');
load('setting.mat','*'); 
model_SS = [var_SS dvar_SS]; % steady states of endogenous variables and derived variables (Phi)
model_SM = model_SS; % smoothed variables
model_SK = shock; % smoothed shocks
if simulation>0
    varnames_END = (SS0.Properties.VariableNames)';
    varnames_END = varnames_END(1:end-size(parnames_DER,1),:); % endogenous variable names
end

P = single(data_OBS.P); % population growth
P(isnan(P)) = nanmean(P);
NNS = zeros(Nperiod,1); % Number of Negative States

% Assigning values for parameters (FIX & TBE)
for ii = 1:size(parnames_FIX,1) % assign par values
    feval(@() assignin('caller',parnames_FIX{ii},par_FIX.(parnames_FIX{ii})))
end
for ii = 1:size(parnames_TBE,1) % assign par values
    feval(@() assignin('caller',parnames_TBE{ii},x(ii)))
    par_TBE.(parnames_TBE{ii}) = x(ii);
end

% For simulations with fixed CES
if simulation==6
    par_END.s(:) = data_s15y.s(1);
    TT=1;
elseif simulation==7
    par_END.s(:) = 1;
    TT=1;
end

% The initial period
for tt = 1:Nperiod
    % Solve the model
    if simulation==0 || (simulation>0 && tt<=TT) % estimation
        PX = data_X.P(tt);
        WX = data_X.W(tt);
        [SS,SSX] = sss(tt,par_FIX,par_TBE,par_EXO,par_END,PX,WX); % calculate the SS for tt
        Xnames = fieldnames(SS);
        for ii = 1:(size(Xnames,1)) % collecting SS
            model_SS{tt,Xnames{ii}} = SS.(Xnames{ii});
        end
        if rundynare==1 % if we want to extract smoothed variables and shocks by running dynare
            Xnames = fieldnames(SSX); % endogenous variable names with X + 2 derived variables
            for ii = 1:size(Xnames,1) % assigning SS values
                feval(@() assignin('caller',Xnames{ii},SSX.(Xnames{ii})))
            end
            run model.m; % run Dynare code
            Xnames = fieldnames(SS); % endogenous variable names w/o X + 2 derived variables
            for ii = 1:(size(Xnames,1)-2) % collecting smoothed variables leaving Phi_n and Phi_q out
                model_SM{tt,Xnames{ii}} = oo_.SmoothedVariables.(Xnames{ii})(tt);
            end
            for ii = 1:size(M_.exo_names,1)
                model_SK{tt,M_.exo_names{ii}} = oo_.SmoothedShocks.(M_.exo_names{ii})(tt);
            end
        end
    end
    if simulation>0 && tt>TT % simulation
        var0 = model_SS{tt-1,1:end-size(parnames_DER,1)}; % initial guess (the original SS)
        fff = @(var)sssm(var,varnames_END,tt,par_FIX,par_TBE,par_SIM.par_EXO,par_END,par_SIM.par_DER);
        options = optimoptions('fsolve','MaxFunctionEvaluations',10000);
        rng('default')
        [model_SS{tt,1:end-size(parnames_DER,1)},~,exitflag] = fsolve(fff,var0,options);
        if exitflag<=0 || model_SS{tt,'B'}>1
            model_SS{tt,1:end-size(parnames_DER,1)} = model_SS{tt-1,1:end-size(parnames_DER,1)};
            NNS(tt) = 1;
        end
        for ii = 1:size(parnames_DER,1)
            model_SS{tt,parnames_DER{ii}} = par_SIM.par_DER.(parnames_DER{ii})(tt);
        end
    end
    % count the number of negative states or impossible CBR
    if simulation==0 && (nnz(model_SS{tt,:}<0)>=1 || model_SS{tt,'q'}>5)
        NNS(tt) = 1; % count the number of negative states or impossible CBR
    end
end

% sign restrictions
for ii = 1:size(parnames_DER,1)
    par_DER.(parnames_DER{ii}) = model_SS.(parnames_DER{ii}); % obtain the derived time-varying parameters
end
for ii = 1:size(parnames_DER,1)
    if sum((par_DER.(parnames_DER{ii}))>0)==Nperiod % only if Phi_n/q are all positive (meaningful)
        xxx = [ones(length(sample.(parnames_DER{ii})),1),data_s15y{sample.(parnames_DER{ii}),parnames_AUX.(parnames_DER{ii})}]; % regressors
        yyy = log(par_DER.(parnames_DER{ii})(sample.(parnames_DER{ii})));
        [coeff,~,resid] = regress(yyy,xxx); % regression coeff and residual
        RMSE = sqrt(sum(resid.^2)/(size(xxx,1)-size(xxx,2))); % standard error of the residuals
        pvalue = 1-tcdf(abs(coeff./(RMSE*sqrt(diag((xxx'*xxx)^(-1))))),size(xxx,1)-size(xxx,2)); % pvalues of one-sided t tests
        if strcmp(parnames_DER{ii},'Phi_n') % sign restrictions in equation of Phi_n
            sign.(parnames_DER{ii}) = ~[coeff([false;strcmp(parnames_AUX.Phi_n,'WP')])<0
                                       coeff([false;strcmp(parnames_AUX.Phi_n,'S')])>0
                                       coeff([false;strcmp(parnames_AUX.Phi_n,'URB')])>0
                                       coeff([false;strcmp(parnames_AUX.Phi_n,'fpratio')])>0
                                       ]; % sign restrictions satisfied = 0
            NSR.(parnames_DER{ii}) = (~(sign.(parnames_DER{ii}))).*(pvalue(2:end)>0.05).*(pvalue(2:end)); % No. of sign restrictions not satisfied weighted by p-values
        elseif strcmp(parnames_DER{ii},'Phi_q') % sign restrictions in equation of Phi_q
            sign.(parnames_DER{ii}) = ~[coeff([false;strcmp(parnames_AUX.Phi_q,'FL')])<0
                                       coeff([false;strcmp(parnames_AUX.Phi_q,'S_insp')])<0
                                       coeff([false;strcmp(parnames_AUX.Phi_q,'URB')])>-inf
                                       ]; % sign restrictions satisfied = 0
            NSR.(parnames_DER{ii}) = (~(sign.(parnames_DER{ii}))).*(pvalue(2:end)>0.05).*(pvalue(2:end)); % No. of sign restrictions not satisfied weighted by p-values
        end
    else
        sign.(parnames_DER{ii}) = 4;
        NSR.(parnames_DER{ii}) = 4; % if not computable, set the NSR of that equation to 3
    end
end

% gap
y_model = model_SS{:,{'A','miu'}};
y_data = data_OBS{:,{'A','miu'}};
y = sum(NNS)*100+(sum(sign.Phi_n)+sum(sign.Phi_q))*100+(sum(NSR.Phi_n)+sum(NSR.Phi_q))*10+mean(nanmean(abs(100*(y_model-y_data)./y_data)));
end

function [yy,yX] = sss(tt,par_FIX,par_TBE,par_EXO,par_END,PX,WX)
% calculate the steady state of the model for period t (given PX and WX)

% assign parameters
parnames_FIX = fieldnames(par_FIX); % extract par names
for ii = 1:size(parnames_FIX,1) % assign fixed par values
    feval(@() assignin('caller',parnames_FIX{ii},par_FIX.(parnames_FIX{ii})))
end
parnames_TBE = fieldnames(par_TBE); % extract par names
for ii = 1:size(parnames_TBE,1) % assign fixed par values
    feval(@() assignin('caller',parnames_TBE{ii},par_TBE.(parnames_TBE{ii})))
end
parnames_EXO = fieldnames(par_EXO); % extract par names
for ii = 1:size(parnames_EXO,1) % assigning time-varying values of mortalities
    feval(@() assignin('caller',parnames_EXO{ii},par_EXO.(parnames_EXO{ii})(tt)))
end
parnames_END = fieldnames(par_END); % extract par names
for ii = 1:size(parnames_END,1) % assigning time-varying values of ces
    feval(@() assignin('caller',parnames_END{ii},par_END.(parnames_END{ii})(tt)))
end

gam = 1-alph-bet;

B = (1-PX)/(m0+(1-m0)*m1/PX+(1-m0)*(1-m1)*m2/(PX^2)+(1-m0)*(1-m1)*(1-m2)*m3/(PX^3)+(1-m0)*(1-m1)*(1-m2)*(1-m3)*(1/4)/(PX^4)+(1-m0)*(1-m1)*(1-m2)*(1-m3)*(1/4)/(PX^5)+(1-m0)*(1-m1)*(1-m2)*(1-m3)*(2/4)/(PX^6)-1);
D = 1+B-PX;
P1 = (1-m0)*(1-m1)*B/PX;
P2 = P1*(1-m2)/PX;
P3 = P2*(1-m3)/PX;
L = P1+P2+P3;
ADR = (((3/4)*P3/PX+(2/4)*P3/(PX*PX)+(1/4)*P3/(PX*PX*PX))/L);
Y = (L^theta1);
w = theta1*Y/L;
H = (WX/(PX^(theta1+theta2-1)))^(1/theta2);
X = (H^theta2)*(PX^(theta1+theta2-1));
Q = H/((P1/L)+(P2/(L*H))+(P3/(L*H*H)));
q = (H^2)*(Q^(epsilon/(1-epsilon)));
M1 = (tau_0+tau_w*(WX-1)+tau_A*a_0/(1-a_A))/(1-tau_miu);
M2 = tau_A*a_b*B/((1-a_A)*((P1+P2)/2)*(1-tau_miu));
miu = (1+M1-sqrt((1+M1)^2-4*(M1+M2)))/2;
b = B/((1-miu)*((P1+P2)/2));
n = b*(1-m0)*(1-m1);
A = exp((a_0+a_b*b)/(1-a_A));
z1 = w/(1+ADR);
CC = m2*z1+(1-m2)*m3*(w+w*X)/(2*(1+ADR))+(1-m2)*(1-m3)*(w+w*X+w*X*X)/(3*(1+ADR));
Cz = ((alph*X/(gam*n))^(1/s))*n/X;
z = CC/(1+Cz);
childcare_cost = z*Cz/(((1-m2)*m3/(2*(1+ADR))+(1-m2)*(1-m3)/(3*(1+ADR)))*X);
z2 = (w+w*X-childcare_cost*X)/(2*(1+ADR));
z3 = (w+w*X+w*X*X-childcare_cost*X)/(3*(1+ADR));
pq = (z/(X*X))*((bet*X/(gam*q))^(1/s))/((1-m2)*m3*b/(2*(1+ADR))+(1-m2)*(1-m3)*b/(3*(1+ADR)));
pn = (childcare_cost-pq*q*b)/b;
Phi_n = pn/w;
Phi_q = pq/w;

yy.P = PX;
yy.W = WX;
yy.B = B;
yy.D = D;
yy.X = X;
yy.P1 = P1;
yy.P2 = P2;
yy.P3 = P3;
yy.L = L;
yy.ADR = ADR;
yy.Y = Y;
yy.w = w;
yy.H = H;
yy.Q = Q;
yy.q = q;
yy.miu = miu;
yy.b = b;
yy.n = n;
yy.A = A;
yy.z1 = z1;
yy.z = z;
yy.z2 = z2;
yy.z3 = z3;
yy.pq = pq;
yy.pn = pn;
yy.Phi_n = Phi_n;
yy.Phi_q = Phi_q;

yX.PX = PX;
yX.WX = WX;
yX.BX = B;
yX.DX = D;
yX.XX = X;
yX.P1X = P1;
yX.P2X = P2;
yX.P3X = P3;
yX.LX = L;
yX.ADRX = ADR;
yX.YX = Y;
yX.wX = w;
yX.HX = H;
yX.QX = Q;
yX.qX = q;
yX.miuX = miu;
yX.bX = b;
yX.nX = n;
yX.AX = A;
yX.z1X = z1;
yX.zX = z;
yX.z2X = z2;
yX.z3X = z3;
yX.pqX = pq;
yX.pnX = pn;
yX.Phi_n = Phi_n;
yX.Phi_q = Phi_q;
end

function F = sssm(var,varnames,tt,par_FIX,par_TBE,par_EXO,par_END,par_DER)
% simulate the steady state of the model for period t

% assign parameters
parnames_FIX = fieldnames(par_FIX); % extract par names
for ii = 1:size(parnames_FIX,1) % assign fixed par values
    feval(@() assignin('caller',parnames_FIX{ii},par_FIX.(parnames_FIX{ii})))
end
parnames_TBE = fieldnames(par_TBE); % extract par names
for ii = 1:size(parnames_TBE,1) % assign fixed par values
    feval(@() assignin('caller',parnames_TBE{ii},par_TBE.(parnames_TBE{ii})))
end
parnames_EXO = fieldnames(par_EXO); % extract par names
for ii = 1:size(parnames_EXO,1) % assigning time-varying values of mortalities
    feval(@() assignin('caller',parnames_EXO{ii},par_EXO.(parnames_EXO{ii})(tt)))
end
parnames_END = fieldnames(par_END); % extract par names
for ii = 1:size(parnames_END,1) % assigning time-varying values of ces
    feval(@() assignin('caller',parnames_END{ii},par_END.(parnames_END{ii})(tt)))
end
parnames_DER = fieldnames(par_DER); % extract par names
for ii = 1:size(parnames_DER,1) % assigning time-varying values of derived parameters
    feval(@() assignin('caller',parnames_DER{ii},par_DER.(parnames_DER{ii})(tt)))
end

gam = 1-alph-bet;

% x -> varnames assignment
for ii = 1:size(varnames,1)
    feval(@() assignin('caller',varnames{ii},var(ii)))
end

F = [(z/(X*X))*(((alph*X*z)/(gam*z*n))^(1/s)) - (((pn+pq*q)/((1-m0)*(1-m1)))*((1-m2)*m3*(1/((1+ADR)+(1+ADR)))+(1-m2)*(1-m3)*(1/((1+ADR)+(1+ADR)+(1+ADR)))));
    (z/(X*X))*(((bet*X*z)/(gam*z*q))^(1/s)) - ((pq*b)*((1-m2)*m3*(1/((1+ADR)+(1+ADR)))+(1-m2)*(1-m3)*(1/((1+ADR)+(1+ADR)+(1+ADR)))));
    w - theta1*Y/L;
    z - (m2*z1+(1-m2)*m3*z2+(1-m2)*(1-m3)*z3);
    (1+ADR)*z1 - w;
    z2*((1+ADR)+(1+ADR))+b*(pn+pq*q)*X - (w+w*X);
    z3*((1+ADR)+(1+ADR)+(1+ADR))+b*(pn+pq*q)*X - (w+w*X+w*X*X);
    Y - (L^theta1);
    P - (1-D+B);
    D - (m0*B+m1*(1-m0)*B/P+m2*P1/P+m3*P2/P+(1/4)*P3/P+(1/4)*P3/(P*P)+(2/4)*P3/(P*P*P));
    B - (1-miu)*((P1+P2)/2)*b;
    P1 - ((1-m0)*(1-m1)*B/P);
    P2 - (P1*(1-m2)/P);
    P3 - (P2*(1-m3)/P);
    ADR - (((3/4)*P3/P+(2/4)*P3/(P*P)+(1/4)*P3/(P*P*P))/L);
    L - (P1+P2+P3);
    H - ((P1/L)*Q+(P2/L)*(Q/H)+(P3/L)*(Q/(H*H)));
    pn - Phi_n*w;
    pq - Phi_q*w;
    miu - (tau_0+tau_miu*miu+tau_A*log(A)+tau_w*(W-1));
    log(A) - (a_0+a_A*log(A)+a_b*b);
    Q - (((Q*q/(H*H))^(1-epsilon)));
    b - n/((1-m0)*(1-m1));
    W - (w*X/w);
    X - (H^theta2)*(P^(theta1+theta2-1))];
end

function y = matchfill(x,short,long)
% Input: a short series x, the short index for x, a long index which includes the short as subset.
% Output: match the two series, return a long series with all matched rows filled with the short series, NA otherwise
[~,~,rowindex] = intersect(short,long); % get the row index for the long series
y = nan(size(long));
y(rowindex) = x;
end

function y = int_ext(X0,Y0,X1,lb,ub,use,method)
% interpolation and extrapolation based on (X0,Y0) onto (X1,Y1)
minY = min(Y0);
maxY = max(Y0);
if strcmp(use,'usemax') % use maximum observation to extrapolate
    range_lb = find(Y0==minY,1,'first'):find(Y0==maxY,1,'last');
elseif strcmp(use,'uselast') % use last observation to extrapolate
    range_lb = find(Y0==minY,1,'first'):find(~isnan(Y0),1,'last');
end
range_nan = find(isnan(Y0));
range = setdiff(range_lb,range_nan);
y = interp1(X0(range),Y0(range),X1,method,'extrap');
y(y<lb) = lb;
y(y>ub) = ub;
end

function y = groupsum(x,group,grouplength)
% calculate the sum of x over group, adjusting for NaN and different group size
% Input: x is a long array, grouped by 'group' vector, with a group length
ggg = unique(group); % unique group identifiers
Ngroup = length(ggg); % number of groups
y = NaN(Ngroup,size(x,2)); % x can be a vector or matrix
for iii = 1:Ngroup
    x_ggg = x(group==ggg(iii),:); % identify the elements in x in group ggg(iii)
    Nobs = sum(~isnan(x_ggg)); % number of non-NaNs
    y(iii,1:size(x,2)) = (grouplength./Nobs).*nansum(x_ggg);
end
end

function y = splice(series,baseline_col)
% splice the 2 columns of series with the baseline_col being the baseline column (1 or 2)
common = sum(~isnan(series),2)==2; % obtain the rows with common support
if baseline_col==1
    other_col = 2;
elseif baseline_col==2
    other_col = 1;
end
cratio = mean(series(common,other_col)./series(common,baseline_col)); % common ratio
y = nan(size(series,1),1);
tobespliced = logical(~isnan(series(:,other_col))-common); % observations in the other col to be spliced
y(tobespliced) = series(tobespliced,other_col)/cratio;
y(~tobespliced) = series(~tobespliced,baseline_col);
end

function [coef,stderr,resid,sigma] = tsls(y,X,Z)
% 2SLS regression 
% y: N-1 regressand
% X: N-k original regressors (should include ones for an intercept)
% Z: N-kk instruments (kk>=k) (should include ones for an intercept)
n = length(y); % Number of observations
k = size(X,2); % Number of the regressors in the original model
kk = size(Z,2); % Number of instruments
df = n - k; % Degree of freedom
PZ = Z*((Z'*Z)\eye(kk))*Z';
Xhat = PZ*X;
coef = (Xhat'*Xhat)\(Xhat'*y);
resid = y-X*coef; % residuals
mse = resid'*resid/df; % mean squared error
sigma = sqrt(mse); % root mean squared error (sigma hat)
var_coef = (mse)*((Xhat'*Xhat)\eye(kk));
stderr  = sqrt(diag(var_coef)); % standard errors
end