//Unified Growth Theory
//By James Foreman-Peck and Peng Zhou

// DECLARATION
var 
A b z z1 z2 z3 n q 
Y w W X
P D B P1 P2 P3 ADR L H 
pn pq miu Q;

varexo 
e_Y e_Q e_pn e_pq e_miu e_A;

parameters 
alph bet theta1 theta2 epsilon
a_0 a_A a_b
tau_0 tau_miu tau_A tau_w
s
m0 m1 m2 m3
sigma_A sigma_Y sigma_pn sigma_pq sigma_miu sigma_Q
PX WX XX
AX bX zX z1X z2X z3X nX qX 
YX wX 
DX BX P1X P2X P3X ADRX LX HX 
pnX pqX miuX QX
Phi_n Phi_q;

for i = 1:length(M_.params)
    deep_parameter_name = M_.param_names(i,:);
    eval(['M_.params(i)  = ' deep_parameter_name ';'])
end

// SPECIFICATION
model;
# gam = 1-alph-bet;

(z(-1)/(X(+1)*X))*(((alph*X*z)/(gam*z(-1)*n))^(1/s)) - (((pn(+1)+pq(+1)*q)/((1-m0)*(1-m1)))*((1-m2)*m3*(1/((1+ADR)+(1+ADR(+1))))+(1-m2)*(1-m3)*(1/((1+ADR)+(1+ADR(+1))+(1+ADR(+2))))));
(z(-1)/(X(+1)*X))*(((bet*X*z)/(gam*z(-1)*q))^(1/s)) - ((pq(+1)*b)*((1-m2)*m3*(1/((1+ADR)+(1+ADR(+1))))+(1-m2)*(1-m3)*(1/((1+ADR)+(1+ADR(+1))+(1+ADR(+2))))));
w - theta1*Y/L;
z - (m2*z1+(1-m2)*m3*z2+(1-m2)*(1-m3)*z3);
(1+ADR)*z1 - w;
z2*((1+ADR)+(1+ADR(+1)))+b*(pn(+1)+pq(+1)*q)*X(+1) - (w+w(+1)*X(+1));
z3*((1+ADR)+(1+ADR(+1))+(1+ADR(+2)))+b*(pn(+1)+pq(+1)*q)*X(+1) - (w+w(+1)*X(+1)+w(+2)*X(+2)*X(+1));
Y - exp(e_Y)*(L^theta1);
P - (1-D+B);
D - (m0*B+m1*(1-m0)*B(-1)/P(-1)+m2*P1(-1)/P(-1)+m3*P2(-1)/P(-1)+(1/4)*P3(-1)/P(-1)+(1/4)*P3(-2)/(P(-1)*P(-2))+(2/4)*P3(-3)/(P(-1)*P(-2)*P(-3)));
B - (1-miu)*((P1+P2)/2)*b(-1);
P1 - ((1-m0)*(1-m1)*B(-1)/P(-1));
P2 - (P1(-1)*(1-m2)/P(-1));
P3 - (P2(-1)*(1-m3)/P(-1));
ADR - (((3/4)*P3(-1)/P(-1)+(2/4)*P3(-2)/(P(-1)*P(-2))+(1/4)*P3(-3)/(P(-1)*P(-2)*P(-3)))/L);
L - (P1+P2+P3);
H - ((P1/L)*Q(-1)+(P2/L)*(Q(-2)/H(-1))+(P3/L)*(Q(-3)/(H(-1)*H(-2))));
pn - Phi_n*w*exp(e_pn);
pq - Phi_q*w*exp(e_pq);
miu - (tau_0+tau_miu*miu(-1)+tau_A*log(A)+tau_w*(W-1)+e_miu);
log(A) - (a_0+a_A*log(A(-1))+a_b*b+e_A);
Q - (exp(e_Q)*((Q(-2)*q(-1)/(H(-1)*H))^(1-epsilon)));
b - n/((1-m0)*(1-m1));
W - (w*X/w(-1));
X - (H^theta2)*(P(-1)^(theta1+theta2-1));
end;

steady_state_model;
P = PX;
W = WX;
X = XX;
B = BX;
D = DX;
P1 = P1X;
P2 = P2X;
P3 = P3X;
L = LX;
ADR = ADRX;
Y = YX;
w = wX;
H = HX;
Q = QX;
q = qX;
miu = miuX;
b = bX;
n = nX;
A = AX;
z1 = z1X;
z = zX;
z2 = z2X;
z3 = z3X;
pq = pqX;
pn = pnX;
end;

options_.noprint = 1;
steady;
check;

shocks;
var e_Y; stderr sigma_Y;
var e_pn; stderr sigma_pn;
var e_pq; stderr sigma_pq;
var e_miu; stderr sigma_miu;
var e_A; stderr sigma_A;
var e_Q; stderr sigma_Q;
end;

varobs P W B D A miu;
calib_smoother(datafile='data_obs.mat') P W B D A miu;
stoch_simul(order=1, irf=40, nograph);